@@ -24,7 +24,7 @@
 ;; definitive answer, and sb-eval always answers with just FUNCTION.
 (defun expect-wild-return-type-p (f)
   (declare (ignorable f))
-  (or #+sb-fasteval (typep f 'sb-interpreter:interpreted-function)))
+  (or #+sb-fasteval (typep f 'sb-kernel:interpreted-function)))
 
 (deftest function-lambda-list.1
     (function-lambda-list 'cl-user::one)
@@ -122,7 +122,7 @@
   t)
 
 (deftest find-source-stuff.4
-    (matchp (car (sb-pcl:generic-function-methods #'cl-user::two)) 4)
+    (matchp (car (sb-mop:generic-function-methods #'cl-user::two)) 4)
   t)
 
 (deftest find-source-stuff.5
@@ -242,6 +242,7 @@
     (matchp-name :function 'cl-user::compile-time-too-fun 28)
   t)
 
+(load "load-test.lisp")
 (deftest find-source-stuff.32
     (matchp-name :function 'cl-user::loaded-as-source-fun 3)
   t)
@@ -293,7 +294,8 @@
 ;;;; Check correctness of DEFTYPE-LAMBDA-LIST.
 (deftype foobar-type
     (&whole w &environment e r1 r2 &optional o &rest rest &key k1 k2 k3)
-  (declare (ignore w e r1 r2 o rest k1 k2 k3))
+  (declare (ignore w e r1 r2 o rest k1 k2 k3)
+           (sb-ext:muffle-conditions sb-kernel:&optional-and-&key-in-lambda-list))
   nil)
 
 (deftest deftype-lambda-list.1
@@ -347,12 +349,6 @@
     (tai '*print-base* :heap '(:space :immobile))
   t)
 
-(deftest allocation-information.2c
-  ;; This is a a test of SBCL genesis that leverages sb-introspect.
-    (tai (sb-kernel::find-fdefn (elt sb-vm:+static-fdefns+ 0))
-         :heap '(:space #+immobile-code :immobile #-immobile-code :static))
-  t)
-
 (deftest allocation-information.3
     (tai 42 :immediate nil)
   t)
@@ -372,12 +368,11 @@
 (deftest* (allocation-information.4
            ;; Ignored as per the comment above, even though it seems
            ;; unlikely that this is the right condition.
-           :fails-on (or :win32 (and :sparc :gencgc)))
+           :fails-on (or :win32 :ppc64 (and :sparc :gencgc)))
     #+gencgc
     (tai (make-list 1) :heap
-         `(:space :dynamic :generation 0 :write-protected nil
-           :boxed t :pinned nil :large nil)
-         :ignore (list :page))
+         `(:space :dynamic :boxed t :large nil)
+         :ignore (list :page :pinned :generation :write-protected))
     #-gencgc
     (tai :cons :heap
          ;; FIXME: Figure out what's the right cheney-result. SPARC at least
@@ -389,7 +384,7 @@
 (setq sb-ext:*evaluator-mode* :compile)
 (sb-ext:defglobal *large-obj* nil)
 
-#+(and gencgc (or x86 x86-64 ppc) (not win32))
+#+(and gencgc (or riscv x86 x86-64 ppc) (not win32))
 (progn
   (setq *print-array* nil)
   (setq *large-obj* (make-array (* sb-vm:gencgc-card-bytes 4)
@@ -420,7 +415,7 @@
          (= (getf props :generation) gen)
          (eq (getf props :boxed :missing) boxedp))))
 #+gencgc
-(deftest* (allocation-information.6 :fails-on (or :sparc))
+(deftest* (allocation-information.6 :fails-on :sbcl)
     ;; Remember, all tests run after all toplevel forms have executed,
     ;; so if this were (DEFGLOBAL *LARGE-CODE* ... ) or something,
     ;; the garbage collection explicitly requested for ALLOCATION-INFORMATION.5
@@ -441,7 +436,7 @@
                ;; but the large object size can be as small as 16K.
                ;; 16K might fit in the free space of an open region,
                ;; and by accident would not go on a large object page.
-               (sb-c:allocate-code-object nil 0
+               (sb-c:allocate-code-object nil 0 0
                 (max (* 4 sb-vm:gencgc-card-bytes) #-64-bit 65536))))
       (declare (notinline format))
       (format (make-string-output-stream) "~%")
@@ -634,15 +629,6 @@
                      (:copier copy-our-struct))
     (a 42 :type fixnum))
 
-  ;; This test doesn't work because the XEP for the out-of-line accessor
-  ;; does not include the type test, and the function gets a signature
-  ;; of (FUNCTION (T) (VALUES FIXNUM &OPTIONAL)). This can easily be fixed
-  ;; by deleting (THE <struct> INSTANCE) from the access form
-  ;; and correspondingly adding a declaration on the type of INSTANCE.
-  ;;
-  ;; Yes, it can be fixed, but it is done this way because it produces
-  ;; smaller code.
-  #+nil
   (deftest function-type+defstruct.1
       (values (type-equal (function-type 'struct-a)
                           (function-type #'struct-a))
@@ -775,24 +761,24 @@
 
 ;;; ASDF.  I can't even.
 (compile 'sb-introspect:map-root)
-(defun count-pointees (x simple &aux (n 0))
-  (sb-introspect:map-root (lambda (obj)
-                            (declare (ignore obj))
-                            (incf n))
-                          x
-                          :simple simple)
-  n)
+(defun list-pointees (x simple)
+  (sb-int:collect ((result))
+    (sb-introspect:map-root (lambda (obj) (result obj))
+                            x :simple simple)
+    (result)))
+(defun count-pointees (x simple)
+  (length (list-pointees x simple)))
 
 ;;; A closure points to its underlying function, all closed-over values,
 ;;; and possibly the closure's name.
 ;;; #'SB-INT:CONSTANTLY-T is a nameless closure over 1 value
-(deftest map-root-closure-un (count-pointees #'SB-INT:CONSTANTLY-T nil) 2)
-;;; (SYMBOL-FUNCTION 'AND) is a named closure over 1 value
+(deftest map-root-closure-unnamed (count-pointees #'sb-int:constantly-t nil) 2)
+;;; (SYMBOL-FUNCTION 'AND) is a named closure over 1 value.
+;;; The closed-over value is AND, and the name of the closure is (:MACRO AND).
 (deftest map-root-closure-named (count-pointees (symbol-function 'and) nil) 3)
 
-;;; GFs point to their layout, implementation, slots,
-;;; and a hash-code, except that 64-bit headers can store the hash code
-;;; in the slot vector's high header bytes.
+;;; GFs point to their layout, implementation function, and slot vector.
+;;; There's also a hash-code which is stored is one of two different ways.
 ;;; However, in either case we expect only 3 referenced objects,
 ;;; because due to a strange design choice in MAP-ROOT,
 ;;; it does not invoke the funarg on fixnums (or characters).
