@@ -516,18 +516,18 @@
   ;; of return values and override the naturalize method to perform
   ;; the sign extension (in compiler/target/c-call.lisp).
   (ecase context
-    ((:normal #-(or alpha x86 x86-64) :result)
+    ((:normal #-(or x86 x86-64) :result)
      (list (if (alien-integer-type-signed type) 'signed-byte 'unsigned-byte)
            (alien-integer-type-bits type)))
-    #+(or alpha x86 x86-64)
+    #+(or x86 x86-64)
     (:result
      (list (if (alien-integer-type-signed type) 'signed-byte 'unsigned-byte)
            (max (alien-integer-type-bits type)
                 sb-vm:n-machine-word-bits)))))
 
 ;;; As per the comment in the :ALIEN-REP method above, this is defined
-;;; elsewhere for alpha and x86oids.
-#-(or alpha x86 x86-64)
+;;; elsewhere for x86oids.
+#-(or x86 x86-64)
 (define-alien-type-method (integer :naturalize-gen) (type alien)
   (declare (ignore type))
   alien)
@@ -1272,22 +1272,10 @@
 (declaim (ftype (sfunction (string hash-table) (or integer null))
                 find-foreign-symbol-in-table))
 (defun find-foreign-symbol-in-table (name table)
-  (let ((extern (extern-alien-name name)))
-    (values
-     (or (gethash extern table)
-         (gethash (concatenate 'base-string "ldso_stub__" extern) table)))))
+  (values (gethash (extern-alien-name name) table)))
 
 (in-package "SB-ALIEN")
 
-;;; *STATIC-FOREIGN-SYMBOLS* are static as opposed to "dynamic" (not
-;;; as opposed to C's "extern"). The table contains symbols known at
-;;; the time that the program was built, but not symbols defined in
-;;; object files which have been loaded dynamically since then.
-#-linkage-table
-(progn
-  (declaim (type hash-table *static-foreign-symbols*))
-  (defvar *static-foreign-symbols* (make-hash-table :test 'equal)))
-
 #+sb-xc
 (defmacro maybe-with-pinned-objects (variables types &body body)
   (declare (ignorable variables types))
