@@ -22,7 +22,7 @@
 (def!struct (defstruct-description
              (:conc-name dd-)
              (:copier nil)
-             #-sb-xc-host (:pure t)
+             (:pure t)
              (:constructor make-defstruct-description (null-lexenv-p name)))
   ;; name of the structure
   (name (missing-arg) :type symbol :read-only t)
@@ -50,6 +50,11 @@
   ;; a list of DEFSTRUCT-SLOT-DESCRIPTION objects for all slots
   ;; (including included ones)
   (slots () :type list)
+  ;; bit mask containing a 1 for each word that the garbage collector must visit
+  ;; (as opposed to a raw slot). Certain slot types (notably fixnum) may have either
+  ;; a 0 or a 1 in the mask because it does not matter if it is seen by GC.
+  ;; Bit index 0 in the mask is the word just after the header, and so on.
+  (bitmap +layout-all-tagged+ :type integer)
   ;; a list of (NAME . INDEX) pairs for accessors of included structures
   (inherited-accessor-alist () :type list)
   ;; number of data words, including the layout itself if the layout
@@ -82,36 +87,18 @@
   ;; PURIFY).
   ;; This is only meaningful if DD-CLASS-P = T.
   (pure nil :type (member t nil)))
-#-sb-fluid (declaim (freeze-type defstruct-description))
+(declaim (freeze-type defstruct-description))
 (!set-load-form-method defstruct-description (:host :xc :target))
 
 ;;;; basic LAYOUT stuff
 
-;;; Note: This bound is set somewhat less than MOST-POSITIVE-FIXNUM
-;;; in order to guarantee that several hash values can be added without
-;;; overflowing into a bignum.
-(defconstant layout-clos-hash-limit (1+ (ash sb-xc:most-positive-fixnum -3))
-  "the exclusive upper bound on LAYOUT-CLOS-HASH values")
-;; This must be DEF!TYPE and not just DEFTYPE because access to slots
-;; of a layout occur "before" the structure definition is made in the
-;; run-the-xc pass, and the source-transform of a slot accessor
-;; wraps (TRULY-THE <type> ...) around %INSTANCE-REF,
-;; so <type> had best be defined at that point.
-(def!type layout-clos-hash () `(integer 0 ,layout-clos-hash-limit))
-(declaim (ftype (sfunction (t) layout-clos-hash) randomish-layout-clos-hash))
-
 ;;; Careful here: if you add more bits, then adjust the bit packing for
 ;;; 64-bit layouts which also store LENGTH + DEPTHOID in the same word.
-;;; When testing these, you usually want to use LAYOUT-%BITS rather than
-;;; LAYOUT-FLAGS, because the compiler will stupidly "unpack" the
-;;; flags by masking out the stuff that aren't the "bits" and then
-;;; mask again.
-(defconstant +custom-gc-scavenge-flag+    #b000001)
-(defconstant +structure-layout-flag+      #b000010)
-(defconstant +pathname-layout-flag+       #b000100)
-(defconstant +layout-layout-flag+         #b001000)
-(defconstant +condition-layout-flag+      #b010000)
-(defconstant +pcl-object-layout-flag+     #b100000)
+(defconstant +structure-layout-flag+         #b00000001)
+(defconstant +pathname-layout-flag+          #b00000010)
+(defconstant +condition-layout-flag+         #b00001000)
+(defconstant +pcl-object-layout-flag+        #b00010000)
+(defconstant sb-vm:lockfree-list-node-flag   #b01000000) ; exported for use in gc-private.h
 
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
@@ -126,10 +113,10 @@
 ;;;
 ;;; FIXME: ...it would be better to automate this, of course...
 
-;;; 64-bit layout %BITS slot:
+;;; 64-bit layout FLAGS slot:
 ;;;
-;;; | 4 bytes  | 24 bits | 8 bits |
-;;; +----------+---------+--------+
+;;; | 4 bytes  | 16 bits | 16 bits |
+;;; +----------+---------+---------+
 ;;;  depthoid    length     flags
 ;;;
 ;;; depthoid is stored as a tagged fixnum in its 4 byte field.
@@ -148,26 +135,23 @@
 
 ;;; 32-bit is not done yet. Three slots are still used, instead of two.
 
-(def!struct (layout
-             #+64-bit
-             ;; Accept a specific subset of keywords
-             (:constructor %make-layout (clos-hash classoid %bits
-                                         &key inherits bitmap info invalid
-                                           depth2-ancestor depth3-ancestor depth4-ancestor))
-             #-64-bit
-             (:constructor make-layout (clos-hash classoid
-                                        &key flags invalid inherits depthoid length info bitmap
-                                          depth2-ancestor depth3-ancestor depth4-ancestor))
-             (:copier nil))
+;;; Maximum value of N in ANCESTOR_N. Couldn't come up with a better name.
+(defconstant sb-c::layout-inherits-max-optimized-depth 5)
+(sb-xc:defstruct (layout (:copier nil)
+                         ;; Parsing DEFSTRUCT uses a temporary layout
+                         (:constructor make-temporary-layout
+                             (clos-hash classoid inherits &aux (invalid nil))))
 
   ;; A packed field containing the DEPTHOID, LENGTH, and FLAGS
-  #+64-bit (%bits 0 :type (signed-byte #.sb-vm:n-word-bits))
+  #+64-bit (flags 0 :type (signed-byte #.sb-vm:n-word-bits))
 
   ;; a union of +something-LAYOUT-FLAG+ bits
   #-64-bit (flags 0 :type word :read-only nil)
 
-  ;; a pseudo-random hash value for use by CLOS.
-  (clos-hash (missing-arg) :type layout-clos-hash) ; this no longer defaults to a random number
+  ;; a quasi-random hash value for use by CLOS. Determine by class-name
+  ;; for classes named by a symbol, otherwise a pseudo-random value.
+  ;; Must be acceptable as an argument to SB-INT:MIX
+  (clos-hash (missing-arg) :type (and fixnum unsigned-byte))
   ;; the class that this is a layout for
   (classoid (missing-arg) :type classoid)
   ;; The value of this slot can be:
@@ -207,10 +191,10 @@
   ;; If this layout has some kind of compiler meta-info, then this is
   ;; it. If a structure, then we store the DEFSTRUCT-DESCRIPTION here.
   (info nil :type (or null defstruct-description))
-  ;; Map of raw slot indices.
-  (bitmap +layout-all-tagged+ :type layout-bitmap)
-  ;; Per-slot comparator for implementing EQUALP.
-  (equalp-tests #() :type simple-vector)
+  ;; EQUALP comparator for two instances with this layout
+  ;; Could be the generalized function, or a type-specific one
+  ;; if the defstruct was compiled in a policy of SPEED 3.
+  (equalp-impl #'equalp-err :type (sfunction (t t) boolean) :read-only t)
   ;; If this layout is for an object of metatype STANDARD-CLASS,
   ;; these are the EFFECTIVE-SLOT-DEFINITION metaobjects.
   (slot-list nil :type list)
@@ -221,65 +205,223 @@
   ;; inherited layouts or 0, only pertinent to structure classoids.
   ;; There is no need to store the layout at depths 0 or 1
   ;; since they're predetermined to be T and STRUCTURE-OBJECT.
-  ;; These 3 slots cause the length of a layout to be exactly 16 words
-  ;; with compact-instance-header, which avoids having to change some
-  ;; logic in 'immobile-space' that assumes this alignment constraint.
-  (depth2-ancestor 0)
-  (depth3-ancestor 0)
-  (depth4-ancestor 0))
+  (ancestor_2 0)
+  (ancestor_3 0)
+  (ancestor_4 0)
+  (ancestor_5 0))
 (declaim (freeze-type layout))
 
-#+64-bit
-(progn
-(defmacro pack-layout-bits (depthoid length flags)
+(defun equalp-err (a b)
+  (bug "EQUALP ~S ~S" a b))
+
+;;; Applicable only if bit-packed (for 64-bit architectures)
+(defmacro pack-layout-flags (depthoid length flags)
   `(logior (ash ,(or depthoid -1) (+ 32 sb-vm:n-fixnum-tag-bits))
-           (ash ,(or length 0) 8)
+           (ash ,(or length 0) 16)
            ,(or flags 0)))
-(defmacro unpack-layout-bits (bits field)
-  (ecase field
-    (:depthoid `(ash ,bits (- -32 sb-vm:n-fixnum-tag-bits)))
-    (:length   `(ldb (byte 24 8) ,bits))
-    (:flags    `(ldb (byte 8 0) ,bits))))
 
-(defmacro make-layout (clos-hash classoid &rest rest &key depthoid length flags &allow-other-keys)
-  (setq rest (copy-list rest))
-  (remf rest :depthoid)
-  (remf rest :length)
-  (remf rest :flags)
-  `(%make-layout ,clos-hash ,classoid (pack-layout-bits ,depthoid ,length ,flags) ,@rest))
+(defmacro type-dd-length (type-name)
+  (dd-length (find-defstruct-description type-name)))
+
+(defmacro get-dsd-index (type-name slot-name)
+  ;; It seems to be an error in CCL to declare something NOTINLINE
+  ;; if it is an unknown function.
+  #+sb-xc (declare (notinline dsd-index)) ; forward reference
+  (dsd-index (find slot-name
+                   (dd-slots (find-defstruct-description type-name))
+                   :key #'dsd-name)))
+
+(defmacro set-bitmap-from-layout (to-layout from-layout)
+  #+sb-xc-host (declare (ignore to-layout from-layout))
+  #-sb-xc-host
+  `(let ((index (type-dd-length layout)))
+     (dotimes (i (layout-bitmap-words ,from-layout))
+       (setf (%raw-instance-ref/word ,to-layout index)
+             (%raw-instance-ref/word ,from-layout index))
+       (incf index))))
+
+;;; See the pictures above DD-BITMAP in src/code/defstruct for the details.
+(defconstant standard-gf-primitive-obj-layout-bitmap
+  #+compact-instance-header  6
+  #-compact-instance-header -4)
 
-(declaim (inline layout-length layout-flags))
 #+sb-xc-host
-(defun layout-depthoid (layout)
-  (unpack-layout-bits (layout-%bits layout) :depthoid))
-(defun layout-length (layout)
-  (unpack-layout-bits (layout-%bits layout) :length))
-(defun layout-flags (layout)
-  (unpack-layout-bits (layout-%bits layout) :flags))
+(defmacro set-layout-inherits (layout inherits)
+  `(setf (layout-inherits ,layout) ,inherits))
+#-sb-xc-host
+(defmacro set-layout-inherits (layout inherits &optional recompute-bitmap)
+  `(let* ((l ,layout) (i ,inherits) (d (length i)))
+     (setf (layout-inherits l) i)
+     (setf (layout-ancestor_2 l) (if (> d 2) (svref i 2) 0)
+           (layout-ancestor_3 l) (if (> d 3) (svref i 3) 0)
+           (layout-ancestor_4 l) (if (> d 4) (svref i 4) 0)
+           (layout-ancestor_5 l) (if (> d 5) (svref i 5) 0))
+     ;; This part is for PCL where a class can forward-reference its superclasses
+     ;; and we only decide at class finalization time whether it is funcallable.
+     ;; Picking the right bitmap could probably be done sooner given the metaclass,
+     ;; but this approach avoids changing how PCL uses MAKE-LAYOUT.
+     ;; The big comment above MAKE-IMMOBILE-FUNINSTANCE in src/code/x86-64-vm
+     ;; explains why we differentiate between SGF and everything else.
+     ,(when recompute-bitmap
+        `(when (find ,(find-layout 'function) i)
+           (let ((bitmap
+                  #+immobile-code ; there are two possible bitmaps
+                  ;; *SGF-WRAPPER* isn't defined as yet, but this is just an s-expression.
+                  (if (or (find sb-pcl::*sgf-wrapper* i) (eq l sb-pcl::*sgf-wrapper*))
+                      standard-gf-primitive-obj-layout-bitmap
+                      +layout-all-tagged+)
+                  ;; there is only one possible bitmap otherwise
+                  #-immobile-code standard-gf-primitive-obj-layout-bitmap))
+             (setf (%raw-instance-ref/word l (type-dd-length layout))
+                   (logand sb-ext:most-positive-word bitmap)))))
+     l))
+(push '("SB-KERNEL" set-layout-inherits) *!removable-symbols*)
 
-(declaim (inline (setf layout-depthoid) (setf layout-length) (setf layout-flags)))
-(defun (setf layout-depthoid) (val layout)
-  (declare (type (integer -1 #x7fff) val))
-  (setf (layout-%bits layout) (logior (ash val (+ 32 sb-vm:n-fixnum-tag-bits))
-                                      (ldb (byte 32 0) (layout-%bits layout)))))
-(defun (setf layout-length) (val layout)
-  (setf (unpack-layout-bits (layout-%bits layout) :length) val))
-(defun (setf layout-flags) (val layout)
-  (setf (unpack-layout-bits (layout-%bits layout) :flags) val))
-) ; end PROGN
-
-#-64-bit
+;;; For lack of any better to place to write up some detail surrounding
+;;; layout creation for structure types, I'm putting here.
+;;; When you issue a DEFSTRUCT at the REPL, there are *three* instances
+;;; of LAYOUT makde for the new structure.
+;;; 1) The first is one associated with a temporary instance of
+;;; structure-classoid used in parsing the DEFSTRUCT form so that
+;;; we don't signal an UNKNOWN-TYPE condition for something like:
+;;;   (defstruct chain (next nil :type (or null chain)).
+;;; The temporary classoid is garbage immediately after parsing
+;;; and is never installed.
+;;; 2) The next is the actual LAYOUT that ends up being registered.
+;;; 3) The third is a layout created when setting the "compiler layout"
+;;; which contains copies of the length/depthoid/inherits etc
+;;; that we compare against the isntalled one to make sure they match.
+;;; The third one also gets thrown away.
+#-sb-xc-host
 (progn
-(defmacro layout-%bits (x) `(layout-flags ,x))
-)
+(defun make-layout (clos-hash classoid
+                    &key (depthoid -1) (length 0) (flags 0)
+                         (inherits #())
+                         (info nil)
+                         (bitmap (if info (dd-bitmap info) 0))
+                         (invalid :uninitialized))
+  (let* ((fixed-words (type-dd-length layout))
+         (bitmap-words (ceiling (1+ (integer-length bitmap)) sb-vm:n-word-bits))
+         (nwords (+ fixed-words bitmap-words))
+         (layout (truly-the layout
+                 #+immobile-space
+                 (sb-vm::alloc-immobile-fixedobj
+                  (1+ nwords)
+                  (logior (ash nwords sb-vm:instance-length-shift)
+                          sb-vm:instance-widetag))
+                 #-immobile-space (%make-instance nwords))))
+    (setf (%instance-layout layout) #.(find-layout 'layout))
+    (setf (layout-flags layout) #+64-bit (pack-layout-flags depthoid length flags)
+                                #-64-bit flags)
+    (setf (layout-clos-hash layout) clos-hash
+          (layout-classoid layout) classoid
+          (layout-invalid layout) invalid)
+    #-64-bit (setf (layout-depthoid layout) depthoid (layout-length layout) length)
+    (setf (layout-info layout) info)
+    (setf (layout-slot-list layout) nil
+          (layout-slot-table layout) #(1 nil))
+    (set-layout-inherits layout inherits)
+    #-sb-xc-host
+    (dotimes (i bitmap-words)
+      (setf (%raw-instance-ref/word layout (+ fixed-words i))
+            (ldb (byte sb-vm:n-word-bits (* i sb-vm:n-word-bits)) bitmap)))
+    (setf (layout-invalid layout) invalid)
+    layout))
+(declaim (inline layout-bitmap-words))
+(defun layout-bitmap-words (layout)
+  (declare (layout layout))
+  (- (%instance-length layout) (type-dd-length layout)))
+(defun layout-bitmap (layout)
+  (acond ((layout-info layout) (dd-bitmap it))
+         ;; Instances lacking DD-INFO are CLOS objects, which can't generally have
+         ;; raw slots, except that funcallable-instances may have 2 raw slots -
+         ;; the trampoline and the layout. The trampoline can have a tag, depending
+         ;; on the platform, and the layout is tagged but a special case.
+         ;; In any event, the bitmap is always 1 word.
+         (t (%raw-instance-ref/signed-word layout (type-dd-length layout))))))
 
-(declaim (inline layout-for-std-class-p))
-(defun layout-for-std-class-p (x)
-  (logtest (layout-%bits x) +pcl-object-layout-flag+))
+;;; The cross-compiler representation of a LAYOUT omits several things:
+;;;   * BITMAP - obtainable via (DD-BITMAP (LAYOUT-INFO layout)).
+;;;     GC wants it in the layout to avoid double indirection.
+;;;   * EQUALP-TESTS - needed only for the target's implementation of EQUALP.
+;;;   * SLOT-TABLE, and SLOT-LIST - used only by the CLOS implementation.
+;;;   * ANCESTOR_N are optimizations for TYPEP.
+;;; So none of those really make sense on the host.
+;;; Also, we eschew the packed representation of length+depthoid+flags.
+;;; FLAGS are not even strictly necessary, since they are for optimizing
+;;; various type checks.
+#+sb-xc-host
+(progn
+  (defstruct (layout (:include structure!object)
+                     (:constructor host-make-layout
+                                   (clos-hash classoid &key info depthoid inherits
+                                                       length flags invalid)))
+    ;; Cross-compiler-only translation from slot index to symbol naming
+    ;; the accessor to call. (Since access by position is not a thing)
+    (index->accessor-map #() :type simple-vector)
+    ;; CLOS-HASH is needed to convert some TYPECASE forms to jump tables.
+    ;; Theoretically we don't need this in the cross-compiler, because the
+    ;; layout has a classoid which has a name which has a known hash.
+    ;; But there's no harm in storing it.
+    (clos-hash nil :type (and sb-xc:fixnum unsigned-byte))
+    (classoid nil :type classoid)
+    (flags 0 :type word)
+    (invalid :uninitialized :type (or cons (member nil t :uninitialized)))
+    (inherits #() :type simple-vector)
+    (depthoid -1 :type layout-depthoid)
+    (length 0 :type layout-length)
+    (info nil :type (or null defstruct-description)))
+  (defun make-layout (&rest args)
+    (let ((args (copy-list args)))
+      (remf args :bitmap)
+      (apply #'host-make-layout args)))
+  (defun make-temporary-layout (clos-hash classoid inherits)
+    (host-make-layout clos-hash classoid :inherits inherits :invalid nil))
+  (defun layout-bitmap (layout)
+    (if (layout-info layout) (dd-bitmap (layout-info layout)) +layout-all-tagged+)))
+
+(defmacro sb-c::layout-nth-ancestor-slot (n)
+  `(case ,n
+     (2 'layout-ancestor_2)
+     (3 'layout-ancestor_3)
+     (4 'layout-ancestor_4)
+     (5 'layout-ancestor_5)))
+
+#+(and (not sb-xc-host) 64-bit)
+;;; LAYOUT-DEPTHOID gets a vop and a stub
+(defmacro layout-length (layout) ; SETFable
+  `(ldb (byte 16 16) (layout-flags ,layout)))
+
+(defconstant layout-flags-mask #xffff) ; "strictly flags" bits from the packed field
+
+;;; Abstract out the differences between {32-bit,64-bit} target and XC layouts.
+;;; FLAGS can't change once assigned.
+(defmacro assign-layout-slots (layout &key depthoid length
+                                           (flags `(layout-flags ,layout)) flagsp)
+  (let ((invalidate-p (and (eql depthoid -1) (not length) (not flagsp))))
+    #+(and 64-bit (not xc-host)) ; packed slot
+    `(setf (layout-flags ,layout)
+           ,(if invalidate-p
+                `(logior (ash -1 (+ 32 sb-vm:n-fixnum-tag-bits))
+                         (ldb (byte 32 0) (layout-flags ,layout)))
+                `(pack-layout-flags ,depthoid ,length
+                                    (logand ,flags layout-flags-mask))))
+    #+(or (not 64-bit) sb-xc-host) ; ordinary slot
+    (if invalidate-p
+        `(setf (layout-depthoid ,layout) -1)
+        `(setf (layout-depthoid ,layout) ,depthoid
+               (layout-length ,layout) ,length
+               (layout-flags ,layout) ,flags))))
+
+;;; True of STANDARD-OBJECT, which include generic functions.
+;;; This one includes any class that mixes in STANDARD-OBJECT.
+(declaim (inline layout-for-pcl-obj-p))
+(defun layout-for-pcl-obj-p (x)
+  (logtest (layout-flags x) +pcl-object-layout-flag+))
 
 (declaim (inline sb-fasl:dumpable-layout-p))
 (defun sb-fasl:dumpable-layout-p (x)
-  (and (typep x 'layout) (not (layout-for-std-class-p x))))
+  (and (typep x 'layout) (not (layout-for-pcl-obj-p x))))
 
 ;;; The CLASSOID structure is a supertype of all classoid types.  A
 ;;; CLASSOID is also a CTYPE structure as recognized by the type
@@ -491,8 +633,8 @@
     (dolist (layout (sort (loop for v being each hash-value
                                 of (classoid-subclasses (find-classoid 't))
                                 collect v)
-                          #'< :key #'sb-kernel::layout-flags))
-      (let ((flags (sb-kernel::layout-flags layout)))
+                          #'< :key #'sb-kernel:layout-flags))
+      (let ((flags (sb-kernel:layout-flags layout)))
         (unless (= flags prev)
           (format t "Layout flags = ~d~%" flags)
           (setq prev flags)))
