@@ -16,7 +16,7 @@
 ;;; A number that can represent an index into a vector, including
 ;;; one-past-the-end
 (deftype array-range ()
-  `(integer 0 ,sb-xc:array-dimension-limit))
+  `(integer 0 ,array-dimension-limit))
 
 ;;; a type used for indexing into sequences, and for related
 ;;; quantities like lengths of lists and other sequences.
@@ -35,13 +35,13 @@
 ;;; MOST-POSITIVE-FIXNUM lets the system know it can increment a value
 ;;; of type INDEX without having to worry about using a bignum to
 ;;; represent the result.
-(def!type index () `(integer 0 (,sb-xc:array-dimension-limit)))
+(def!type index () `(integer 0 (,array-dimension-limit)))
 
 ;;; like INDEX, but augmented with -1 (useful when using the index
 ;;; to count downwards to 0, e.g. LOOP FOR I FROM N DOWNTO 0, with
 ;;; an implementation which terminates the loop by testing for the
 ;;; index leaving the loop range)
-(def!type index-or-minus-1 () `(integer -1 (,sb-xc:array-dimension-limit)))
+(def!type index-or-minus-1 () `(integer -1 (,array-dimension-limit)))
 
 ;;; The smallest power of two that is equal to or greater than X.
 (declaim (inline power-of-two-ceiling))
@@ -277,7 +277,7 @@
                        ,result)))
     `(let ((,n-table ,table))
        ,(if locked
-            `(with-locked-system-table (,n-table) ,iter-form)
+            `(with-system-mutex ((hash-table-lock ,n-table)) ,iter-form)
             iter-form))))
 
 ;;; Executes BODY for all entries of PLIST with KEY and VALUE bound to
@@ -285,6 +285,7 @@
 (defmacro doplist ((key val) plist &body body)
   (with-unique-names (tail)
     `(let ((,tail ,plist) ,key ,val)
+       (declare (ignorable ,key ,val))
        (loop (when (null ,tail) (return nil))
              (setq ,key (pop ,tail))
              (when (null ,tail)
@@ -1126,7 +1127,7 @@ NOTE: This interface is experimental and subject to change."
     ((or symbol cons)
      (%check-deprecated-type type-specifier))
     (class
-     (let ((name (class-name type-specifier)))
+     (let ((name (cl:class-name type-specifier)))
        (when (and name (symbolp name)
                   (eq type-specifier (find-class name nil)))
          (%check-deprecated-type name))))))
@@ -1216,7 +1217,9 @@ NOTE: This interface is experimental and subject to change."
                                 ((or (listp id) ; must be a type-specifier
                                      (memq id '(special ignorable ignore
                                                 dynamic-extent
-                                                truly-dynamic-extent))
+                                                truly-dynamic-extent
+                                                sb-c::constant-value
+                                                sb-c::no-constraints))
                                      (info :type :kind id))
                                  (cdr decl))))))
            (partition (spec)
@@ -1335,33 +1338,6 @@ NOTE: This interface is experimental and subject to change."
                                 (length cache))))
                   short-name))))))
 
-;;; Just like WITH-OUTPUT-TO-STRING but doesn't close the stream,
-;;; producing more compact code.
-(defmacro with-simple-output-to-string
-    ((var &optional string (element-type :default)) &body body)
-  ;; Don't need any fancy type-specifier parsing. Uses of this macro
-  ;; are confined to our own code. So the element-type is literally
-  ;; either :DEFAULT or BASE-CHAR.
-  (aver (member element-type '(base-char :default)))
-  (multiple-value-bind (forms decls) (parse-body body nil)
-    (if string
-        `(let ((,var (make-fill-pointer-output-stream ,string)))
-           ,@decls
-           ,@forms)
-        ;; Why not dxify this stream?
-        `(let ((,var #+sb-xc-host (make-string-output-stream)
-                     #-sb-xc-host
-                     ,(case element-type
-                        ;; Non-unicode doesn't have %MAKE-DEFAULT-STRING-OSTREAM
-                        #+sb-unicode
-                        (:default '(%make-default-string-ostream))
-                        ;; and this macro is never employed to make streams
-                        ;; that can only return CHARACTER string.
-                        (t '(%make-base-string-ostream)))))
-           ,@decls
-           ,@forms
-           (get-output-stream-string ,var)))))
-
 (in-package "SB-KERNEL")
 
 (defun fp-zero-p (x)
