@@ -12,14 +12,13 @@
 (in-package "SB-IMPL")
 
 (defmacro with-finalizer-store ((var) &body body)
-  `(let ((mutex (hash-table-lock (finalizer-id-map **finalizer-store**))))
-     ;; This does not inhibit GC, though the hashtable operations will,
-     ;; as is (currently) required for tables with non-null weakness.
-     (sb-thread::with-recursive-system-lock (mutex)
-       ;; Grab the store again inside the lock in case the array was enlarged
-       ;; after we referenced the mutex but before we acquired it.
-       (let ((,var **finalizer-store**))
-         ,@body))))
+  `(with-system-mutex ((hash-table-lock (finalizer-id-map **finalizer-store**)))
+     ;; Grab the global var inside the lock in case the array was enlarged
+     ;; after we referenced the mutex but before we acquired it.
+     ;; It's OK to reference the FINALIZER-ID-MAP because that is always
+     ;; in element 1 of the array regardless of what happens to the array.
+     (let ((,var **finalizer-store**))
+       ,@body)))
 
 (defmacro finalizer-recycle-bin (store) `(cdr (elt ,store 0)))
 (defmacro finalizer-id-map (store) `(elt ,store 1))
@@ -27,9 +26,8 @@
 
 (defun make-finalizer-store (array-length)
   (let* ((v (make-array (the index array-length)))
-         (ht (make-hash-table :test 'eq :weakness :key)))
-    (setf (%instance-ref ht (get-dsd-index hash-table flags))
-          (logior (hash-table-flags ht) hash-table-finalizer-flag))
+         (ht (make-system-hash-table :test 'eq :weakness :key :synchronized nil
+                                     :finalizer t)))
     ;; The recycle bin has a dummy item in front so that the simple-vector
     ;; is growable without messing up RUN-PENDING-FINALIZERS when it atomically
     ;; pushes items into the recycle bin - it is unaffected by looking at
@@ -45,7 +43,8 @@
 (declaim (simple-vector **finalizer-store**))
 
 (defun finalize (object function &key dont-save
-                        &aux (item (if dont-save (list function) function)))
+                        &aux (function (%coerce-callable-to-fun function))
+                             (item (if dont-save (list function) function)))
   "Arrange for the designated FUNCTION to be called when there
 are no more references to OBJECT, including references in
 FUNCTION itself.
@@ -160,7 +159,9 @@ Examples:
   ;; finalizers can in practice almost never be run, as pseudo-static objects
   ;; don't die, making this more-or-less an exercise in futility.
   (with-finalizer-store (old-store)
-    (without-gcing
+    ;; This doesn't need WITHOUT-GCING. MAPHASH will never present its funarg
+    ;; with a culled entry. GC during the MAPHASH could remove some items
+    ;; before we get to them, and that's fantastic.
       (let ((new-store
               (make-finalizer-store (max (1+ (finalizer-max-id old-store))
                                          +finalizers-initial-size+)))
@@ -183,7 +184,7 @@ Examples:
                  old-objects)
         (clrhash old-objects)
         (fill old-store 0)
-        (setq **finalizer-store** new-store)))))
+        (setq **finalizer-store** new-store))))
 
 ;;; Replace the finalizer store with a copy.  Tenured (gen6 = pseudo-static)
 ;;; vectors are problematic in many ways for gencgc, unless immutable.
@@ -233,7 +234,10 @@ Examples:
     object))
 
 ;;; Drain the queue of finalizers and return when empty.
-;;; Concurrent invocations of this function are ok.
+;;; Concurrent invocations of this function in different threads are ok.
+;;; Nested invocations (from a GC forced by a finalizer) are not ok.
+;;; See the trace at the bottom of this file.
+(defvar *in-a-finalizer* nil)
 (defun scan-finalizers ()
   ;; This never acquires the finalizer store lock. Code accordingly.
   (let ((hashtable (finalizer-id-map **finalizer-store**)))
@@ -241,6 +245,8 @@ Examples:
      (let ((cell (hash-table-culled-values hashtable)))
        ;; This is like atomic-pop, but its obtains the first cons cell
        ;; in the list, not the car of the first cons.
+       ;; Possible TODO: when no other work remains, free the *JOINABLE-THREADS*,
+       ;; though MAKE-THREAD and JOIN-THREAD do that also, so there's no memory leak.
        (loop (unless cell (return-from scan-finalizers))
              (let ((actual (cas (hash-table-culled-values hashtable)
                                 cell (cdr cell))))
@@ -252,7 +258,7 @@ Examples:
               ;; so always load the vector again before dereferencing.
               (store **finalizer-store**)
               ;; I don't think we need a barrier; this has a data dependency
-              ;; on (CAR CELL) and STORE. (Alpha with threads, anyone?)
+              ;; on (CAR CELL) and STORE.
               (finalizers (svref store id))) ; [1] load
          (setf (svref store id) 0)           ; [2] store
          ;; The ID can be reused right away. Link it into the recycle list,
@@ -265,7 +271,7 @@ Examples:
          ;; Now call the function(s)
          (flet ((call (finalizer)
                   (let ((fun (if (consp finalizer) (car finalizer) finalizer)))
-                    (handler-case (funcall fun)
+                    (handler-case (let ((*in-a-finalizer* t)) (funcall fun))
                       (error (c)
                         (warn "Error calling finalizer ~S:~%  ~S" fun c))))))
            (if (simple-vector-p finalizers)
@@ -308,8 +314,12 @@ Examples:
   (when (hash-table-culled-values (finalizer-id-map **finalizer-store**))
     (cond #+sb-thread
           ((%instancep *finalizer-thread*)
-           (sb-thread::with-system-mutex (*finalizer-queue-lock*)
-             (sb-thread:condition-notify *finalizer-queue*)))
+           ;; Do absolutely nothing if in a finalizer. If we are then it's obvious
+           ;; that we're in the SCAN-FINALIZERS loop, and that the loop will just
+           ;; go on looping without further provocation.
+           (unless *in-a-finalizer*
+             (with-system-mutex (*finalizer-queue-lock*)
+               (sb-thread:condition-notify *finalizer-queue*))))
           #+sb-thread
           ((eq *finalizer-thread* t) ; Create a new thread
            (sb-thread::make-ephemeral-thread
@@ -324,7 +334,7 @@ Examples:
                 (loop
                   (scan-finalizers)
                   ;; Wait for a notification
-                  (sb-thread::with-system-mutex (*finalizer-queue-lock*)
+                  (with-system-mutex (*finalizer-queue-lock*)
                     ;; Don't go to sleep if *FINALIZER-THREAD* became NIL
                     (unless *finalizer-thread*
                       (return))
@@ -342,6 +352,23 @@ Examples:
 ;;; Make no attempt to drain the queue of pending finalizers.
 ;;; (When called from EXIT, the user must invoke a final GC if there is
 ;;; an expectation that GC-based things run. Similarly when saving a core)
+;;; There's a data race involved during SAVE-LISP-AND-DIE (or SB-POSIX:FORK)
+;;; because *FINALIZER-THREAD* is assigned by that thread itself which means that
+;;; it has to run before we can detect that it exists.
+;;; There are few remedies involving some combination of these steps:
+;;; - Assign a THREAD instance into *finalizer-thread* before it shows
+;;;   up in *ALL-THREADS*. If *FINALIZER-THREAD* has a value but is not in
+;;;   *ALL-THREADS*, then it may be blocked on the mutex. Can we terminate it?
+;;; - During save, acquire the *MAKE-THREAD-LOCK* to get a consistent view of
+;;;   which threads are starting and running.
+;;; - the :pauseless-threadstart code adds new threads to *ALL-THREADS*
+;;;   before starting the posix thread. We could look there.
+;;;   The important thing is to avoid thinking that there is a thread running
+;;;   when it doesn't have a posix thread ID yet.
+;;; - Does acquiring the *MAKE-THREAD-LOCK* before assigning *FINALIZER-THREAD*
+;;;   help anything? Perhaps.
+;;; But first I'd like to entirely eliminate the non-pauseless-threadstart code
+;;; so that we don't have to reason about two totally different solutions.
 (defun finalizer-thread-stop ()
   #+sb-thread
   (let ((thread *finalizer-thread*))
@@ -352,6 +379,9 @@ Examples:
     ;; If it was NIL, it is latched at that state, so we can ignore it.
     (when (null thread)
       (return-from finalizer-thread-stop))
+    ;; If user calls SAVE-LISP-AND-DIE or SB-POSIX:FORK in a finalizer...
+    (when (eq sb-thread:*current-thread* *finalizer-thread*)
+      (error "Can't stop finalizer thread from inside finalizer thread"))
     (let ((oldval (cas *finalizer-thread* thread nil)))
       ;; If GC had started the thread, and it got to its main body only after
       ;; we observed *FINALIZER-THREAD* = T, then we could see a transition
@@ -367,6 +397,141 @@ Examples:
     (when (%instancep thread)
       ;; The finalizer thread will exit when we wake it up
       ;; and it sees that *FINALIZER-THREAD* is NIL.
-      (sb-thread::with-system-mutex (*finalizer-queue-lock*)
+      (with-system-mutex (*finalizer-queue-lock*)
         (sb-thread:condition-notify *finalizer-queue*))
       (sb-thread:join-thread thread)))) ; wait for it
+
+#|
+;;; This is a display produced by annotating parts of gc-common.c and
+;;; interrupt.c with each thread's output in its own column.
+;;; The main thread is on the right.
+
+;;; This output shows that if the finalizer thread calls a function that
+;;; triggers a GC, the interrupt nesting depth can grow without limit.
+;;; The finalizer thread does not have to be a memory hog - it just has
+;;; to be unlucky enough to be the thread that triggers the collection.
+;;; _Any_ thread can bring the GC trigger up to the threshold,
+;;; and as long as the finalizer thread is the one to cross the
+;;; the threshold, it is tasked with triggering the next scan
+;;; of finalizers, but it MUST NOT do so recursively.
+;;;
+;;; The same thing can happen without using a finalizer thread,
+;;; but it's actually easier to understand the output this way.
+
+Thread 2                                     Main
+--------                                     --------
+|                                            Enter SB-EXT:GC 0
+|                                            Stopping world
+|                                            Stopped world
+|                                            SUB-GC calling gc(0)
+|                                            set auto_gc_trig=4858833
+|                                            completed GC
+|                                            Restarted world
+|                                            SB-EXT:GC calling POST-GC
+|                                            ENTER run-pending-finalizers
+|                                            Enter SB-EXT:GC 0
+|                                            Stopping world
+|                                            Stopped world
+|                                            SUB-GC calling gc(6)
+|                                            set auto_gc_trig=5075473
+|                                            completed GC
+|                                            Restarted world
+|                                            SB-EXT:GC calling POST-GC
+|                                            ENTER run-pending-finalizers
+|                                            Trying to start finalizer thread
+| starting
+|                                            Enter SB-EXT:GC 0
+|                                            Stopping world
+| Caught SIGUSR2, pc=52a946bd
+| STOP_FOR_GC PA=1 inh=N sigmask=0
+| Caught SIGILL, pc=52a946cc code 0x9
+| evt 1 >handle_pending
+| STOP_FOR_GC PA=0 inh=N sigmask=0
+| fake ffcall "stop_for_gc"
+| bind(free_ICI, 1)
+|                                            Stopped world
+|                                            SUB-GC calling gc(6)
+|                                            set auto_gc_trig=520eeb3
+|                                            completed GC
+|                                            Restarted world
+|                                            SB-EXT:GC calling POST-GC
+|                                            ENTER run-pending-finalizers
+|                                            Notify finalizer thread
+| unbind free_ICI -> 0
+| leave STOP_FOR_GC
+| evt 1 <handle_pending
+| gc_trig=52bcf80, setting PA-int
+| Caught SIGILL, pc=52a946cc code 0x9
+| evt 2 >handle_pending
+| ENTER maybe_gc (pc was 52a946ce)
+| fake ffcall "maybe_gc"
+| bind(free_ICI, 1)
+| maybe_gc calling SUB-GC
+| Stopping world
+|                                            Caught SIGUSR2, pc=7ff010deef47
+|                                            STOP_FOR_GC PA=0 inh=N sigmask=0
+|                                            fake ffcall "stop_for_gc"
+|                                            bind(free_ICI, 1)
+| Stopped world
+| SUB-GC calling gc(0)
+| set auto_gc_trig=552b033
+| completed GC
+| Restarted world
+| maybe_gc calling POST-GC
+| ENTER run-pending-finalizers
+|                                            unbind free_ICI -> 0
+|                                            leave STOP_FOR_GC
+| gc_trig=55d9110, setting PA-int
+| Caught SIGILL, pc=52a946cc code 0x9
+| evt 3 >handle_pending
+| ENTER maybe_gc (pc was 52a946ce)
+| fake ffcall "maybe_gc"
+| bind(free_ICI, 2)
+| maybe_gc calling SUB-GC
+| Stopping world
+|                                            Caught SIGUSR2, pc=7ff010deef47
+|                                            STOP_FOR_GC PA=0 inh=N sigmask=0
+|                                            fake ffcall "stop_for_gc"
+|                                            bind(free_ICI, 1)
+| Stopped world
+| SUB-GC calling gc(0)
+| set auto_gc_trig=56c0423
+| completed GC
+| Restarted world
+| maybe_gc calling POST-GC
+| ENTER run-pending-finalizers
+|                                            unbind free_ICI -> 0
+|                                            leave STOP_FOR_GC
+| gc_trig=576e500, setting PA-int
+| Caught SIGILL, pc=52a946cc code 0x9
+| evt 4 >handle_pending
+| ENTER maybe_gc (pc was 52a946ce)
+| fake ffcall "maybe_gc"
+| bind(free_ICI, 3)
+| maybe_gc calling SUB-GC
+| Stopping world
+|                                            Caught SIGUSR2, pc=7ff010deef47
+|                                            STOP_FOR_GC PA=0 inh=N sigmask=0
+|                                            fake ffcall "stop_for_gc"
+|                                            bind(free_ICI, 1)
+| Stopped world
+| SUB-GC calling gc(0)
+| set auto_gc_trig=59dc213
+| completed GC
+| Restarted world
+| maybe_gc calling POST-GC
+| ENTER run-pending-finalizers
+|                                            unbind free_ICI -> 0
+|                                            leave STOP_FOR_GC
+| gc_trig=5a8a2f0, setting PA-int
+| Caught SIGILL, pc=52a946cc code 0x9
+| evt 5 >handle_pending
+| ENTER maybe_gc (pc was 52a946ce)
+| fake ffcall "maybe_gc"
+| bind(free_ICI, 4)
+| maybe_gc calling SUB-GC
+| Stopping world
+|
+;;; This pattern of binding FREE_INTERRUPT_CONTEXT_INDEX to successively
+;;; higher values will continue forever until reaching the limit and crashing.
+|#
