@@ -147,7 +147,13 @@ statistics are appended to it."
 
 ;;; For GENCGC all generations < GEN will be GC'ed.
 
-(define-load-time-global *already-in-gc* (sb-thread:make-mutex :name "GC lock"))
+(defmacro try-acquire-gc-lock (&rest forms)
+  #-sb-thread `(progn ,@forms t)
+  #+sb-thread
+  `(when (eql (alien-funcall (extern-alien "try_acquire_gc_lock" (function int))) 1)
+     ,@forms
+     (alien-funcall (extern-alien "release_gc_lock" (function void)))
+     t))
 
 (defun sub-gc (gen)
   (cond (*gc-inhibit*
@@ -193,8 +199,6 @@ statistics are appended to it."
            ;; Let's make sure we're not interrupted and that none of
            ;; the deadline or deadlock detection stuff triggers.
            (without-interrupts
-             (sb-thread::without-thread-waiting-for
-                 (:already-without-interrupts t)
                (let ((sb-impl::*deadline* nil)
                      (epoch *gc-epoch*))
                  (loop
@@ -208,10 +212,9 @@ statistics are appended to it."
                     ;; execute the remainder of the GC: stopping the
                     ;; world with interrupts disabled is the mother of
                     ;; all critical sections.
-                    (cond ((sb-thread:with-mutex (*already-in-gc* :wait-p nil)
+                    (cond ((try-acquire-gc-lock
                              (unsafe-clear-roots gen)
-                             (gc-stop-the-world)
-                             t)
+                             (gc-stop-the-world))
                            ;; Success! GC.
                            (perform-gc)
                            ;; Return, but leave *gc-pending* as is: we
@@ -238,7 +241,7 @@ statistics are appended to it."
                   ;; runtime.
                   (when (and (eql gen 0)
                              (neq epoch *gc-pending*))
-                    (return 0))))))))))
+                    (return 0)))))))))
 
 (defun post-gc ()
   ;; Outside the mutex, interrupts may be enabled: these may cause
@@ -251,8 +254,12 @@ statistics are appended to it."
   ;;
   ;; KLUDGE: Don't run the hooks in GC's if:
   ;;
-  ;; A) this thread is dying, so that user-code never runs with
-  ;;    (thread-alive-p *current-thread*) => nil
+  ;; A) this thread is dying or just born, so that user-code never runs with
+  ;;    (thread-alive-p *current-thread*) => nil.
+  ;;    The just-born case can happen with foreign threads that are unlucky
+  ;;    enough to be elected to perform GC just as they begin executing
+  ;;    ENTER-FOREIGN-CALLBACK. This definitely seems to happen with sb-safepoint.
+  ;;    I'm not sure whether it can happen without sb-safepoint.
   ;;
   ;; B) interrupts are disabled somewhere up the call chain since we
   ;;    don't want to run user code in such a case.
@@ -263,7 +270,10 @@ statistics are appended to it."
   ;; but it's not permissible to invoke CONDITION-NOTIFY from a
   ;; dying thread, so we still need the guard for that, but not
   ;; the guard for whether interupts are enabled.
-  (when (sb-thread:thread-alive-p sb-thread:*current-thread*)
+  (when (and
+         #+sb-thread (/= 0 (sap-int (sb-vm::current-thread-offset-sap
+                                     sb-vm::thread-lisp-thread-slot)))
+         (sb-thread:thread-alive-p sb-thread:*current-thread*))
     #+sb-thread (alien-funcall (extern-alien "empty_thread_recyclebin" (function void)))
     (let ((threadp #+sb-thread (%instancep sb-impl::*finalizer-thread*)))
       (when threadp
