@@ -193,7 +193,7 @@ variable: an unreadable object representing the error is printed instead.")
      (multiple-value-bind (fun pretty)
          (and *print-pretty* (pprint-dispatch object))
        (if pretty
-           (with-simple-output-to-string (stream)
+           (%with-output-to-string (stream)
               (sb-pretty::with-pretty-stream (stream)
                 (funcall fun stream object)))
            (let ((buffer-size (approx-chars-in-repr object)))
@@ -206,7 +206,7 @@ variable: an unreadable object representing the error is printed instead.")
                                (finite-base-string-output-stream-pointer stream)))))))
     ;; Could do something for other numeric types, symbols, ...
     (t
-     (with-simple-output-to-string (stream)
+     (%with-output-to-string (stream)
        (output-object object stream)))))
 
 ;;; Estimate the number of chars in the printed representation of OBJECT.
@@ -218,7 +218,7 @@ variable: an unreadable object representing the error is printed instead.")
   ;; This is exact for bases which are exactly a power-of-2, or an overestimate
   ;; otherwise, as mandated by the finite output stream.
   (let ((bits-per-char
-         (aref #.(sb-xc:coerce
+         (aref #.(coerce
                   ;; base 2 or base 3  = 1 bit per character
                   ;; base 4 .. base 7  = 2 bits per character
                   ;; base 8 .. base 15 = 3 bits per character, etc
@@ -488,16 +488,19 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;; For each character, the value of the corresponding element is the
 ;;; lowest base in which that character is a digit.
-(defconstant +digit-bases+
+(defconstant-eqx +digit-bases+
   #.(let ((a (sb-xc:make-array 128 ; FIXME
+                               :retain-specialization-for-after-xc-core t
                                :element-type '(unsigned-byte 8)
                                :initial-element 36)))
       (dotimes (i 36 a)
         (let ((char (digit-char i 36)))
-          (setf (aref a (sb-xc:char-code char)) i)))))
+          (setf (aref a (sb-xc:char-code char)) i))))
+  #'equalp)
 
-(defconstant +character-attributes+
+(defconstant-eqx +character-attributes+
   #.(let ((a (sb-xc:make-array 160 ; FIXME
+                               :retain-specialization-for-after-xc-core t
                                :element-type '(unsigned-byte 16)
                                :initial-element 0)))
       (flet ((set-bit (char bit)
@@ -529,7 +532,8 @@ variable: an unreadable object representing the error is printed instead.")
         (dotimes (i 160) ; FIXME
           (when (zerop (aref a i))
             (setf (aref a i) funny-attribute))))
-      a))
+      a)
+  #'equalp)
 
 ;;; A FSM-like thingie that determines whether a symbol is a potential
 ;;; number or has evil characters in it.
