@@ -312,7 +312,7 @@
                                        (lvar-value element-type))))
                            (cond
                              ((or (null ctype) (contains-unknown-type-p ctype)) '*)
-                             (t (sb-xc:upgraded-array-element-type
+                             (t (upgraded-array-element-type
                                  (lvar-value element-type))))))
                         (t
                          '*))
@@ -405,7 +405,7 @@
   ;; optimizing `#(#(foo bar) #(,x ,y)) we convert the whole expression
   ;; into (VECTOR 'FOO 'BAR X Y), whereas in the unidimensional case
   ;; it never makes sense to turn #(FOO BAR) into (VECTOR 'FOO 'BAR).
-  (when (or (and (= rank 1) (sb-xc:constantp initial-contents env))
+  (when (or (and (= rank 1) (constantp initial-contents env))
             ;; If you inhibit inlining these - game over.
             (fun-lexically-notinline-p 'vector env)
             (fun-lexically-notinline-p 'list env)
@@ -413,12 +413,12 @@
     (return-from rewrite-initial-contents (values nil nil)))
   (let ((dimensions (make-array rank :initial-element nil))
         (output))
-    (named-let recurse ((form (handler-case (sb-xc:macroexpand initial-contents env)
+    (named-let recurse ((form (handler-case (macroexpand initial-contents env)
                                 (error ()
                                   (return-from rewrite-initial-contents))))
                         (axis 0))
       (flet ((make-list-ctor (tail &optional (prefix nil prefixp) &aux val)
-               (when (and (sb-xc:constantp tail)
+               (when (and (constantp tail)
                           (or (proper-list-p (setq val (constant-form-value tail env)))
                               (and (vectorp val) (not prefixp))))
                  (setq form
@@ -435,7 +435,7 @@
       (unless (and (typep form '(cons (member list vector)))
                    (do ((items (cdr form))
                         (length 0 (1+ length))
-                        (fun (let ((axis (the (mod #.sb-xc:array-rank-limit) (1+ axis))))
+                        (fun (let ((axis (the (mod #.array-rank-limit) (1+ axis))))
                                (if (= axis rank)
                                    (lambda (item) (push item output))
                                    (lambda (item) (recurse item axis))))))
@@ -458,16 +458,16 @@
      ;; If the unaltered INITIAL-CONTENTS were constant, then the flattened
      ;; form must be too. Turning it back to a self-evaluating object
      ;; is essential to avoid compile-time blow-up on huge vectors.
-     (if (sb-xc:constantp initial-contents env)
+     (if (constantp initial-contents env)
          (map 'vector (lambda (x) (constant-form-value x env)) output)
          (let ((f (if (singleton-p output) 'list 'vector)))
            `(locally (declare (notinline ,f))
              (,f ,@(mapcar (lambda (x)
                              (cond ((and (symbolp x)
                                          (not (nth-value
-                                               1 (sb-xc:macroexpand-1 x env))))
+                                               1 (macroexpand-1 x env))))
                                     x)
-                                   ((sb-xc:constantp x env)
+                                   ((constantp x env)
                                     `',(constant-form-value x env))
                                    (t
                                     `(locally (declare (inline ,f)) ,x))))
@@ -484,13 +484,13 @@
 ;;; in this path, mainly due to complications in picking the right widetag.
 (define-source-transform make-array (dims-form &rest rest &environment env
                                                &aux dims dims-constp)
-  (cond ((and (sb-xc:constantp dims-form env)
+  (cond ((and (constantp dims-form env)
               (proper-list-p (setq dims (constant-form-value dims-form env)))
               (not (singleton-p dims))
               (every (lambda (x) (typep x 'index)) dims))
          (setq dims-constp t))
         ((and (cond ((typep (setq dims (handler-case
-                                           (sb-xc:macroexpand dims-form env)
+                                           (macroexpand dims-form env)
                                          (error ()
                                            (return-from make-array (values nil t)))))
                             '(cons (eql list)))
@@ -499,7 +499,7 @@
                     ;; `(,X 2 1) -> (LIST* X '(2 1)) for example
                     ((typep dims '(cons (eql list*) cons))
                      (let ((last (car (last dims))))
-                       (when (sb-xc:constantp last env)
+                       (when (constantp last env)
                          (let ((lastval (constant-form-value last env)))
                            (when (listp lastval)
                              (setq dims (append (butlast (cdr dims)) lastval))
@@ -507,7 +507,7 @@
               (proper-list-p dims)
               (not (singleton-p dims)))
          ;; If you spell '(2 2) as (LIST 2 2), it is constant for purposes of MAKE-ARRAY.
-         (when (every (lambda (x) (sb-xc:constantp x env)) dims)
+         (when (every (lambda (x) (constantp x env)) dims)
            (let ((values (mapcar (lambda (x) (constant-form-value x env)) dims)))
              (when (every (lambda (x) (typep x 'index)) values)
                (setq dims values dims-constp t)))))
@@ -539,7 +539,7 @@
                     (case k
                       (:element-type
                        (unless (eq et unsupplied) (return nil))
-                       (setq et (car v) et-constp (sb-xc:constantp et env)))
+                       (setq et (car v) et-constp (constantp et env)))
                       (:initial-element
                        (when (or contents element) (return nil))
                        (setq element v))
@@ -608,7 +608,7 @@
                  `(fill-array ,(car contents) ,alloc-form)))))))
 
 (define-source-transform coerce (x type &environment env)
-  (if (and (sb-xc:constantp type env)
+  (if (and (constantp type env)
            (proper-list-p x)
            (memq (car x) '(sb-impl::|List| list
                            sb-impl::|Vector| vector)))
@@ -874,9 +874,10 @@
                   ;; didn't realize this.
                   (cond
                     (initial-element
-                     (compiler-warn "~S ~S is not a ~S"
-                                    :initial-element default-initial-element
-                                    elt-spec))
+                     (compiler-warn 'array-initial-element-mismatch
+                                    :format-control "~S ~S is not a ~S"
+                                    :format-arguments
+                                    (list :initial-element default-initial-element elt-spec)))
                     ;; For the default initial element, only warn if
                     ;; any array elements are initialized using it.
                     ((and (not (eql c-length 0))
@@ -884,9 +885,9 @@
                           ;; then fill-array means it was supplied initial-contents
                           (not (lvar-matches-calls (combination-lvar call)
                                                    '(make-array-header* fill-array))))
-                     (compiler-style-warn "The default initial element ~S is not a ~S."
-                                          default-initial-element
-                                          elt-spec))))
+                     (compiler-style-warn 'initial-element-mismatch-style-warning
+                                          :format-control "The default initial element ~S is not a ~S."
+                                          :format-arguments (list default-initial-element elt-spec)))))
              (let ((lambda-list `(length ,@(eliminate-keywords))))
                `(lambda ,lambda-list
                   (declare (ignorable ,@lambda-list))
@@ -995,8 +996,9 @@
                    ((not (ctypep value eltype-type))
                     ;; this case will not cause an error at runtime, but
                     ;; it's still worth STYLE-WARNing about.
-                    (compiler-style-warn "~S is not a ~S."
-                                         value eltype)))))
+                    (compiler-style-warn 'initial-element-mismatch-style-warning
+                                         :format-control "~S is not a ~S."
+                                         :format-arguments (list value eltype))))))
              `(let ((array ,creation-form))
                 (multiple-value-bind (vector)
                     (%data-vector-and-index array 0)
@@ -1075,7 +1077,7 @@
                                  'simple-array)
                             ,(cond ((null element-type) t)
                                    (element-type-ctype
-                                    (sb-xc:upgraded-array-element-type
+                                    (upgraded-array-element-type
                                      (lvar-value element-type)))
                                    (t '*))
                             ,(make-list rank :initial-element '*))))
@@ -1357,12 +1359,11 @@
 ;;; compile-time constant.
 (deftransform vector-length ((vector))
   (let* ((vtype (lvar-type vector))
-         (dim (first (array-type-dimensions-or-give-up vtype))))
-    (when (eq dim '*)
+         (dim (array-type-dimensions-or-give-up vtype)))
+    (when (or (not (typep dim '(cons integer)))
+              (conservative-array-type-complexp vtype))
       (give-up-ir1-transform))
-    (when (conservative-array-type-complexp vtype)
-      (give-up-ir1-transform))
-    dim))
+    (first dim)))
 
 (defoptimizer (vector-length derive-type) ((vector))
   (let ((array-type (lvar-conservative-type vector))
@@ -1426,15 +1427,18 @@
              (ref-p array-ref)
              (ref-p index-ref)
              (or
-              (let ((index-leaf (ref-leaf index-ref)))
+              (let* ((index-leaf (ref-leaf index-ref))
+                     (index-value (and (constant-p index-leaf)
+                                       (constant-value index-leaf)))
+                     (index-value (and (integerp index-value)
+                                       index-value)))
                 (loop for constraint in (ref-constraints array-ref)
                       for y = (constraint-y constraint)
                       thereis (and
                                (eq (constraint-kind constraint) 'array-in-bounds-p)
-                               (if (constant-p index-leaf)
+                               (if index-value
                                    (and (constant-p y)
-                                        (<= (constant-value index-leaf)
-                                            (constant-value y)))
+                                        (<= index-value (constant-value y)))
                                    (eq index-leaf y)))))
               (loop for constraint in (ref-constraints index-ref)
                     thereis (and (eq (constraint-kind constraint) 'array-in-bounds-p)
