@@ -19,7 +19,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; miscellaneous backend properties
 
@@ -31,14 +31,14 @@
 ;;; the byte order of the target machine. :BIG-ENDIAN has the MSB first (e.g.
 ;;; IBM RT), :LITTLE-ENDIAN has the MSB last (e.g. DEC VAX).
 (defglobal *backend-byte-order*
-  #!+little-endian :little-endian
-  #!+big-endian :big-endian)
+  #+little-endian :little-endian
+  #+big-endian :big-endian)
 (declaim (type (member nil :little-endian :big-endian) *backend-byte-order*))
 
 ;;; translation from SC numbers to SC info structures. SC numbers are always
 ;;; used instead of names at run time, so changing this vector changes all the
 ;;; references.
-(defglobal *backend-sc-numbers* (make-array sb!vm:sc-number-limit :initial-element nil))
+(defglobal *backend-sc-numbers* (make-array sb-vm:sc-number-limit :initial-element nil))
 (declaim (type sc-vector *backend-sc-numbers*))
 
 ;;; a vector of all the SBs defined, so that we can easily iterate over them
@@ -46,17 +46,16 @@
 (declaim (type simple-vector *backend-sbs*))
 
 ;;; translation from template names to template structures
-(defglobal *backend-template-names* (make-hash-table :test 'eq))
+(defglobal *backend-template-names* (make-hash-table)) ; keys are symbols
 (declaim (type hash-table *backend-template-names*))
 
 ;;; hashtables mapping from SC and SB names to the corresponding structures
-(defglobal *backend-sc-names* (make-hash-table :test 'eq))
+(defglobal *backend-sc-names* (make-hash-table))
 (declaim (type hash-table *backend-sc-names*))
 
 ;;; translations from primitive type names to the corresponding
 ;;; primitive-type structure.
-(defglobal *backend-primitive-type-names*
-  (make-hash-table :test 'eq))
+(defglobal *backend-primitive-type-names* (make-hash-table)) ; keys are symbols
 (declaim (type hash-table *backend-primitive-type-names*))
 
 ;;; This establishes a convenient handle on primitive type unions, or
@@ -81,13 +80,17 @@
 
 ;;; a hashtable translating from VOP names to the corresponding VOP-PARSE
 ;;; structures. This information is only used at meta-compile time.
-(defglobal *backend-parsed-vops* (make-hash-table :test 'eq))
+(defglobal *backend-parsed-vops* (make-hash-table)) ; keys are symbols
 (declaim (type hash-table *backend-parsed-vops*))
 
+;;; Mapping of predicate name (a symbol) to a type.
+;;; This is an EQL table so that it uses symbol-hash.
+(defglobal *backend-predicate-types* (make-hash-table))
 ;;; mappings between CTYPE structures and the corresponding predicate.
 ;;; The type->predicate mapping is implemented as an alist because
 ;;; there is no such thing as a TYPE= hash table.
-(defglobal *backend-predicate-types* (make-hash-table :test 'eq))
+;;; (But if we could make all the types use as keys in this table
+;;; into interned types, could it become an EQ table perhaps?)
 (defglobal *backend-type-predicates* nil)
 (declaim (type hash-table *backend-predicate-types*))
 (declaim (type list *backend-type-predicates*))
@@ -150,10 +153,10 @@ SPARC code in CMUCL,
 and at the IR2 translation stage, the function #'`(LAMBDA () ,GUARD) would be called.
 
 Until SBCL-0.7pre57, this is translated as
-  (:GUARD #!+(OR :SPARC-V8 (AND :SPARC-V9 (NOT :SPARC-64))) T
-          #!-(OR :SPARC-V8 (AND :SPARC-V9 (NOT :SPARC-64))) NIL)
+  (:GUARD #+(OR :SPARC-V8 (AND :SPARC-V9 (NOT :SPARC-64))) T
+          #-(OR :SPARC-V8 (AND :SPARC-V9 (NOT :SPARC-64))) NIL)
 which means that whether this VOP will ever be used is determined at
-compiler compile-time depending on the contents of SB!XC:*FEATURES*.
+compiler compile-time depending on the contents of SB-XC:*FEATURES*.
 
 As of SBCL-0.7pre57, a new special variable,
 SB-C:*BACKEND-SUBFEATURES*?, is introduced. As of that version, only
