@@ -38,7 +38,7 @@
 ;;; Check everything that we can think of for consistency. When a
 ;;; definite inconsistency is detected, we BARF.
 ;;; Our argument is a list of components, but
-;;; we also look at the *FREE-VARS*, *FREE-FUNS* and *CONSTANTS*.
+;;; we also look at the *IR1-NAMESPACE*.
 ;;;
 ;;; First we do a pre-pass which finds all the CBLOCKs and CLAMBDAs,
 ;;; testing that they are linked together properly and entering them
@@ -46,7 +46,7 @@
 ;;; the actual code and control flow. Finally, we scan the global leaf
 ;;; hashtables, looking for lossage.
 (declaim (ftype (function (list) (values)) check-ir1-consistency))
-(defun check-ir1-consistency (components)
+(defun check-ir1-consistency (components &aux (ns *ir1-namespace*))
   (let ((*seen-blocks* (make-hash-table :test 'eq))
         (*seen-funs* (make-hash-table :test 'eq)))
     (unwind-protect
@@ -69,16 +69,15 @@
                  (unless (or (eq block tail)
                              (eq (block-component block) c))
                    (barf "~S is not in ~S." block c)))
-               #|
-               (when (or (loop-blocks c) (loop-inferiors c))
-               (do-blocks (block c :both)
-               (setf (block-flag block) nil))
-               (check-loop-consistency c nil)
-               (do-blocks (block c :both)
-               (unless (block-flag block)
-               (barf "~S was not in any loop." block))))
-               |#
-               ))
+               (let ((component-outer-loop (component-outer-loop c)))
+                 (when (or (loop-blocks component-outer-loop)
+                           (loop-inferiors component-outer-loop))
+                   (do-blocks (block c :both)
+                     (setf (block-flag block) nil))
+                   (check-loop-consistency component-outer-loop nil)
+                   (do-blocks (block c :both)
+                     (unless (block-flag block)
+                       (barf "~S was not in any loop." block)))))))
            (check-fun-consistency components)
 
            (dolist (c components)
@@ -93,32 +92,32 @@
                                   (and (global-var-p v)
                                        (member (global-var-kind v)
                                                '(:global :special :unknown))))
-                        (barf "strange *FREE-VARS* entry: ~S" v))
+                        (barf "strange FREE-VARS entry: ~S" v))
                       (when (leaf-p v)
                         (dolist (n (leaf-refs v))
                           (check-node-reached n)))
                       (when (basic-var-p v)
                         (dolist (n (basic-var-sets v))
                           (check-node-reached n))))
-                    *free-vars*)
+                    (free-vars ns))
 
            (maphash (lambda (k v)
                       (declare (ignore k))
                       (unless (constant-p v)
-                        (barf "strange *CONSTANTS* entry: ~S" v))
+                        (barf "strange CONSTANTS entry: ~S" v))
                       (dolist (n (leaf-refs v))
                         (check-node-reached n)))
-                    *constants*)
+                    (eq-constants ns))
 
            (maphash (lambda (k v)
                       (declare (ignore k))
                       (unless (or (functional-p v)
                                   (and (global-var-p v)
                                        (eq (global-var-kind v) :global-function)))
-                        (barf "strange *FREE-FUNS* entry: ~S" v))
+                        (barf "strange FREE-FUNS entry: ~S" v))
                       (dolist (n (leaf-refs v))
                         (check-node-reached n)))
-                    *free-funs*))
+                    (free-funs ns)))
       (clrhash *seen-blocks*)
       (clrhash *seen-funs*))
     (values)))
@@ -244,12 +243,11 @@
 
 ;;;; loop consistency checking
 
-#|
 ;;; Descend through the loop nesting and check that the tree is well-formed
 ;;; and that all blocks in the loops are known blocks. We also mark each block
 ;;; that we see so that we can do a check later to detect blocks that weren't
 ;;; in any loop.
-(declaim (ftype (function (loop (or loop null)) (values)) check-loop-consistency))
+(declaim (ftype (function (cloop (or cloop null)) (values)) check-loop-consistency))
 (defun check-loop-consistency (loop superior)
   (unless (eq (loop-superior loop) superior)
     (barf "wrong superior in ~S, should be ~S" loop superior))
@@ -278,7 +276,7 @@
   (values))
 
 ;;; Check that Block is either in Loop or an inferior.
-(declaim (ftype (function (block loop) (values)) check-loop-block))
+(declaim (ftype (function (cblock cloop) (values)) check-loop-block))
 (defun check-loop-block (block loop)
   (unless (gethash block *seen-blocks*)
     (barf "unseen block ~S in loop info for ~S" block loop))
@@ -291,8 +289,6 @@
       (barf "~S is in loop info for ~S but not in the loop." block loop)))
   (values))
 
-|#
-
 ;;; Check a block for consistency at the general flow-graph level, and
 ;;; call CHECK-NODE-CONSISTENCY on each node to locally check for
 ;;; semantic consistency.
@@ -760,14 +756,8 @@
 (defun check-block-conflicts (component)
   (do-ir2-blocks (block component)
     (do ((conf (ir2-block-global-tns block)
-               (global-conflicts-next-blockwise conf))
-         (prev nil conf))
+               (global-conflicts-next-blockwise conf)))
         ((null conf))
-      (when prev
-        (unless (> (tn-number (global-conflicts-tn conf))
-                   (tn-number (global-conflicts-tn prev)))
-          (barf "~S and ~S out of order in ~S" prev conf block)))
-
       (unless (find-in #'global-conflicts-next-tnwise
                        conf
                        (tn-global-conflicts
@@ -906,6 +896,9 @@
                           '((setq num (string->id num))))
                       ;; numbers start from 1, not 0
                       (and (< (1- num) (length array)) (aref array (1- num)))))))))
+  (def objmap-id-to-node node-id id-node node)
+  (def objmap-id-to-comp component-id id-component component)
+  (def objmap-id-to-leaf leaf-id id-leaf leaf)
   (def objmap-id-to-cont %cont-num num-cont (or ctran lvar))
   (def objmap-id-to-ir2block ir2-block-id id-ir2-block ir2-block)
   (def objmap-id-to-tn tn-id id-tn tn)
@@ -945,7 +938,7 @@
     (integer (ctran-block (num-cont thing)))
     (functional (lambda-block (main-entry thing)))
     (null (error "Bad thing: ~S." thing))
-    (symbol (block-or-lose (gethash thing *free-funs*)))))
+    (symbol (block-or-lose (gethash thing (free-funs *ir1-namespace*))))))
 
 ;;; Print cN.
 (defun print-ctran (cont)
@@ -1098,7 +1091,7 @@
 ;;; necessary.
 (defun print-vop (vop)
   (pprint-logical-block (*standard-output* nil)
-    (princ (vop-info-name (vop-info vop)))
+    (princ (vop-name vop))
     (princ #\space)
     (pprint-indent :current 0)
     (print-operands (vop-args vop))
@@ -1108,7 +1101,7 @@
        (multiway-branch-if-eq
         (princ (vop-codegen-info vop)))
        (t
-        (princ (with-simple-output-to-string (stream)
+        (princ (%with-output-to-string (stream)
                  ;; Current print depth varies based on whether PRINT-VOP
                  ;; is called by DESCRIBE-IR2-COMPONENT or TRACE-INSTRUCTION,
                  ;; so any fixed value of *PRINT-LEVEL* changes its effect
