@@ -26,25 +26,18 @@
 (declaim (freeze-type opaque-box))
 
 ;;; ANSI limits on compilation
-(defconstant sb-xc:call-arguments-limit sb-xc:most-positive-fixnum
+(defconstant call-arguments-limit most-positive-fixnum
   "The exclusive upper bound on the number of arguments which may be passed
   to a function, including &REST args.")
-(defconstant sb-xc:lambda-parameters-limit sb-xc:most-positive-fixnum
+(defconstant lambda-parameters-limit most-positive-fixnum
   "The exclusive upper bound on the number of parameters which may be specified
   in a given lambda list. This is actually the limit on required and &OPTIONAL
   parameters. With &KEY and &AUX you can get more.")
-(defconstant sb-xc:multiple-values-limit sb-xc:most-positive-fixnum
+(defconstant multiple-values-limit most-positive-fixnum
   "The exclusive upper bound on the number of multiple VALUES that you can
   return.")
 
-;;;; cross-compiler-only versions of CL special variables, so that we
-;;;; don't have weird interactions with the host compiler
 
-(defvar sb-xc:*compile-file-pathname*)
-(defvar sb-xc:*compile-file-truename*)
-(defvar sb-xc:*compile-print*)
-(defvar sb-xc:*compile-verbose*)
-
 ;;;; miscellaneous types used both in the cross-compiler and on the target
 
 ;;;; FIXME: The INDEX and LAYOUT-DEPTHOID definitions probably belong
@@ -56,10 +49,7 @@
 ;;; otherwise a slot unto itself.
 (def!type layout-depthoid () '(integer -1 #x7FFF))
 (def!type layout-length () '(integer 0 #xFFFF))
-(def!type layout-bitmap ()
-  ;; FIXME: Probably should exclude negative bignum
-  #+compact-instance-header 'integer
-  #-compact-instance-header '(and integer (not (eql 0))))
+(def!type layout-bitmap () 'integer)
 
 ;;; An INLINEP value describes how a function is called. The values
 ;;; have these meanings:
@@ -408,18 +398,6 @@ the stack without triggering overflow protection.")
 (sb-impl::define-thread-local *compilation*)
 (declaim (type compilation *compilation*))
 
-(in-package "SB-ALIEN")
-
-;;; Information describing a heap-allocated alien.
-(def!struct (heap-alien-info (:copier nil))
-  ;; The type of this alien.
-  (type (missing-arg) :type alien-type)
-  ;; Its name.
-  (alien-name (missing-arg) :type simple-string)
-  ;; Data or code?
-  (datap (missing-arg) :type boolean))
-(!set-load-form-method heap-alien-info (:xc :target))
-
 ;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
 ;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
 #+linux ; shadow space differs by OS
