@@ -84,6 +84,10 @@
                          sb-vm:bignum-digits-offset
                          index offset))
 
+(defun dd-contains-raw-slots-p (dd)
+  (dolist (dsd (dd-slots dd))
+    (unless (eq (dsd-raw-type dsd) t) (return t))))
+
 (deftransform copy-structure ((instance) * * :result result :node node)
   (let* ((classoid (lvar-type instance))
          (name (and (structure-classoid-p classoid) (classoid-name classoid)))
@@ -104,8 +108,8 @@
                  ;; Also note that VAR-ALLOC can not cope with dynamic-extent except where
                  ;; support has been added (x86oid so far); so requiring an exact type here
                  ;; causes VAR-ALLOC to become FIXED-ALLOC which works on more architectures.
-                 #-c-stack-is-control-stack
-                 (and dd (eql (layout-bitmap layout) sb-kernel:+layout-all-tagged+))
+                 #-c-stack-is-control-stack (and dd (not (dd-contains-raw-slots-p dd)))
+
                  ;; Definitely do this if copying to stack
                  (or (lvar-dynamic-extent result)
                      ;; Or if it's a small fixed number of words
@@ -117,7 +121,6 @@
     ;; There are some benefits to using the simple case for a known exact type:
     ;; - the layout can be wired in which might or might not save 1 instruction
     ;;   depending on whether layouts are in immobile space.
-    ;; - all backends can do this (subject to stack-allocatable-fixed-objects)
     ;; - for a small number of slots, copying them is inlined
     (cond ((not dd) ; it's going to be some subtype of NAME
            `(%copy-instance (%make-instance (%instance-length instance)) instance))
@@ -132,19 +135,24 @@
           (t
            `(%copy-instance-slots (%make-structure-instance ,dd nil) instance)))))
 
+(defun varying-length-struct-p (classoid)
+  ;; This is a nice feature to have in general, but at present it is only possible
+  ;; to make varying length instances of LAYOUT and nothing else.
+  (eq (classoid-name classoid) 'layout))
+
 (deftransform %instance-length ((instance))
   (let ((classoid (lvar-type instance)))
     (if (and (structure-classoid-p classoid)
              (sb-kernel::compiler-layout-ready-p (classoid-name classoid))
              (eq (classoid-state classoid) :sealed)
+             (not (varying-length-struct-p classoid))
              ;; TODO: if sealed with subclasses which add no slots, use the fixed length
              (not (classoid-subclasses classoid)))
         (dd-length (layout-info (sb-kernel::compiler-layout-or-lose (classoid-name classoid))))
         (give-up-ir1-transform))))
 
-;;; The layout is stored in slot 0.
-;;; *** These next two transforms should be the only code, aside from
-;;;     some parts of the C runtime, with knowledge of the layout index.
+;;; *** These transforms should be the only code, aside from the C runtime
+;;;     with knowledge of the layout index.
 #+compact-instance-header
 (define-source-transform function-with-layout-p (x) `(functionp ,x))
 #-compact-instance-header
@@ -154,11 +162,7 @@
   (define-source-transform %set-instance-layout (x val)
     `(%instance-set ,x 0 (the layout ,val)))
   (define-source-transform function-with-layout-p (x)
-    `(funcallable-instance-p ,x))
-  (define-source-transform %fun-layout (x)
-    `(truly-the layout (%funcallable-instance-info ,x 0)))
-  (define-source-transform %set-funcallable-instance-layout (x val)
-    `(setf (%funcallable-instance-info ,x 0) (the layout ,val))))
+    `(funcallable-instance-p ,x)))
 
 ;;;; simplifying HAIRY-DATA-VECTOR-REF and HAIRY-DATA-VECTOR-SET
 
@@ -385,7 +389,7 @@
                 ,(if (and (integer-type-p index-type)
                           (numeric-type-low index-type))
                      `(integer ,(numeric-type-low index-type)
-                               (,sb-xc:array-dimension-limit))
+                               (,array-dimension-limit))
                      `index))))))
 
 (deftransform %data-vector-and-index ((%array %index)
@@ -505,6 +509,7 @@
                   (word-logical-not (%vector-raw-bits bit-array index))))))))
 
 (deftransform bit-vector-= ((x y) (simple-bit-vector simple-bit-vector))
+  ;; TODO: unroll if length is known and not more than a few words
   `(and (= (length x) (length y))
         (let ((length (length x)))
           (or (= length 0)
@@ -545,6 +550,7 @@
 
 (deftransform count ((item sequence) (bit simple-bit-vector) *
                      :policy (>= speed space))
+  ;; TODO: unroll if length is known and not more than a few words
   `(let ((length (length sequence)))
     (if (zerop length)
         0
