@@ -158,13 +158,11 @@
         ;; thread->alloc_region.free_pointer
         (free-pointer
          #+sb-thread (thread-slot-ea thread-alloc-region-slot)
-         #-sb-thread (ea (+ static-space-start
-                            (ash vector-data-offset word-shift))))
+         #-sb-thread (ea boxed-region))
         ;; thread->alloc_region.end_addr
         (end-addr
          #+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
-         #-sb-thread (ea (+ static-space-start
-                            (ash (1+ vector-data-offset) word-shift)))))
+         #-sb-thread (ea (+ boxed-region n-word-bytes))))
 
     (cond ((typep size `(integer , large-object-size))
            ;; large objects will never be made in a per-thread region
@@ -383,9 +381,9 @@
            (length :scs (any-reg immediate))
            (words :scs (any-reg immediate)))
     (:results (result :scs (descriptor-reg) :from :load))
-    (:temporary (:sc any-reg :offset ecx-offset :from :eval) rcx)
-    (:temporary (:sc any-reg :offset eax-offset :from :eval) rax)
-    (:temporary (:sc any-reg :offset edi-offset :from :eval) rdi)
+    (:temporary (:sc any-reg :offset rcx-offset :from :eval) rcx)
+    (:temporary (:sc any-reg :offset rax-offset :from :eval) rax)
+    (:temporary (:sc any-reg :offset rdi-offset :from :eval) rdi)
     (:temporary (:sc complex-double-reg) zero)
     (:arg-types positive-fixnum positive-fixnum positive-fixnum)
     (:translate allocate-vector)
@@ -472,8 +470,7 @@
                                (ash 1 (1+ (- word-shift n-fixnum-tag-bits)))))
                      ,answer)))
            (compute-end ()
-             `(let ((size (cond ((or (not (fixnump size))
-                                     (plausible-signed-imm32-operand-p size))
+             `(let ((size (cond ((typep size '(or (signed-byte 32) tn))
                                  size)
                                 (t
                                  (inst mov limit size)
@@ -593,12 +590,6 @@
 
 ;;;; automatic allocators for primitive objects
 
-(define-vop (make-unbound-marker)
-  (:args)
-  (:results (result :scs (descriptor-reg any-reg)))
-  (:generator 1
-    (inst mov result unbound-marker-widetag)))
-
 (define-vop (make-funcallable-instance-tramp)
   (:args)
   (:results (result :scs (any-reg)))
@@ -615,18 +606,12 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
-   (let* ((instancep (typep type 'layout)) ; is this any instance?
-          (layoutp #+immobile-space
-                   (eq type #.(find-layout 'layout))) ; " a new LAYOUT instance?
+   (let* ((instancep (typep type 'layout)) ; is this an instance type?
           (bytes (pad-data-block words)))
     (progn name) ; possibly not used
-    (unless (or stack-allocate-p layoutp)
+    (unless stack-allocate-p
       (instrument-alloc bytes node))
     (pseudo-atomic (:elide-if stack-allocate-p)
-     (cond (layoutp
-            (invoke-asm-routine 'call 'alloc-layout node)
-            (inst mov result r11-tn))
-           (t
             ;; If storing a header word, defer ORing in the lowtag until after
             ;; the header is written so that displacement can be 0.
             (allocation nil bytes (if type 0 lowtag) node stack-allocate-p result)
@@ -641,7 +626,7 @@
                     ;; where this instruction must write exactly 4 bytes.
                     (inst mov :dword (ea 0 result) header)
                     (storew* header result 0 0 (not stack-allocate-p)))
-                (inst or :byte result lowtag))))))
+                (inst or :byte result lowtag))))
     (when instancep ; store its layout
       (inst mov :dword (ea (+ 4 (- lowtag)) result)
             (make-fixup type :layout))))))
@@ -691,22 +676,26 @@
                                            temp-reg-tn)))))))
 #+immobile-space
 (define-vop (alloc-immobile-fixedobj)
-  (:info lowtag size header)
-  (:temporary (:sc unsigned-reg :to :eval :offset rdi-offset) c-arg1)
-  (:temporary (:sc unsigned-reg :to :eval :offset rsi-offset) c-arg2)
+  (:args (size-class :scs (any-reg) :target c-arg1)
+         (nwords :scs (any-reg) :target c-arg2)
+         (header :scs (any-reg) :target c-arg3))
+  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval :offset rdi-offset) c-arg1)
+  (:temporary (:sc unsigned-reg :from (:argument 1) :to :eval :offset rsi-offset) c-arg2)
+  (:temporary (:sc unsigned-reg :from (:argument 2) :to :eval :offset rdx-offset) c-arg3)
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset)
               c-result)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
-   (inst mov c-arg1 size)
-   (inst mov c-arg2 header)
+   (inst mov c-arg1 size-class)
+   (inst mov c-arg2 nwords)
+   (inst mov c-arg3 header)
    ;; RSP needn't be restored because the allocators all return immediately
    ;; which has that effect
    (inst and rsp-tn -16)
    (pseudo-atomic ()
-     (c-call "alloc_fixedobj")
-     (inst lea result (ea lowtag c-result)))))
+     (c-call "alloc_immobile_fixedobj")
+     (move result c-result))))
 
 (define-vop (alloc-dynamic-space-code)
   (:args (total-words :scs (signed-reg) :target c-arg1))
