@@ -54,7 +54,7 @@
   (define-gprs t *qword-regs* +qword-register-names+
     #("RAX" "RCX" "RDX" "RBX" "RSP" "RBP" "RSI" "RDI"
       "R8"  "R9"  "R10" "R11" "R12" "R13" "R14" "R15"))
-  (define-gprs t *dword-regs* +dword-register-names+
+  (define-gprs nil *dword-regs* +dword-register-names+
     #("EAX" "ECX" "EDX"  "EBX"  "ESP"  "EBP"  "ESI"  "EDI"
       "R8D" "R9D" "R10D" "R11D" "R12D" "R13D" "R14D" "R15D"))
   (define-gprs nil *word-regs* +word-register-names+
@@ -401,7 +401,7 @@
 ;;; the appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
   (typecase value
-    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
+    ((or (integer #.most-negative-fixnum #.most-positive-fixnum)
          character)
      immediate-sc-number)
     (symbol ; Symbols in static and immobile space are immediate
