@@ -45,6 +45,12 @@
    :references '((:amop :generic-function allocate-instance)
                  (:amop :function set-funcallable-instance-function))))
 
+(defmacro error-no-implementation-function (fin)
+  `(error 'unset-funcallable-instance-function
+          :format-control "~@<The function of funcallable instance ~
+                           ~S has not been set.~@:>"
+          :format-arguments (list ,fin)))
+
 (defun allocate-standard-funcallable-instance (wrapper name)
   (declare (layout wrapper))
   (let* ((hash (if name
@@ -66,14 +72,10 @@
                                          (%make-standard-funcallable-instance slots hash))))
                        (setf (%fun-layout f) wrapper)
                        f)))))
-    (set-funcallable-instance-function
-     fin
-     #'(lambda (&rest args)
-         (declare (ignore args))
-         (error 'unset-funcallable-instance-function
-                :format-control "~@<The function of funcallable instance ~
-                                 ~S has not been set.~@:>"
-                :format-arguments (list fin))))
+    (setf (%funcallable-instance-fun fin)
+          (lambda (&rest args)
+            (declare (ignore args))
+            (error-no-implementation-function fin)))
     fin))
 
 (defun classify-slotds (slotds)
@@ -171,6 +173,9 @@
           (multiple-value-bind (slots cpl default-initargs direct-subclasses)
               (!early-collect-inheritance name)
             (let* ((class (find-class name))
+                   (bitmap (if (memq name '(standard-generic-function))
+                               sb-kernel::standard-gf-primitive-obj-layout-bitmap
+                               +layout-all-tagged+))
                    (wrapper (cond ((eq class slot-class)
                                    slot-class-wrapper)
                                   ((eq class standard-class)
@@ -194,7 +199,7 @@
                                   ((eq class standard-generic-function)
                                    standard-generic-function-wrapper)
                                   (t
-                                   (!boot-make-wrapper (length slots) name))))
+                                   (!boot-make-wrapper (length slots) name bitmap))))
                    (proto nil))
               (let ((symbol (make-class-symbol name)))
                 (when (eq (info :variable :kind symbol) :global)
@@ -565,7 +570,7 @@
 
 (defun class-of (x)
   (declare (explicit-check))
-  (wrapper-class* (layout-of x)))
+  (wrapper-class (layout-of x)))
 
 (defun eval-form (form)
   (lambda () (eval form)))
@@ -659,7 +664,8 @@
       (set-layout-inherits layout
                            (order-layout-inherits
                             (map 'simple-vector #'class-wrapper
-                                 (reverse (rest (class-precedence-list class))))))
+                                 (reverse (rest (class-precedence-list class)))))
+                           t)
       (register-layout layout :invalidate t)
 
       ;; FIXME: I don't think this should be necessary, but without it
