@@ -244,49 +244,44 @@ void gc_mark_range(lispobj* where, long count) {
 #define HT_ENTRY_LIVENESS_FUN_ARRAY_NAME alivep_funs
 #include "weak-hash-pred.inc"
 
+static void trace_using_layout(lispobj layout, lispobj* where, int nslots)
+{
+    // Apart from the allowance for untagged pointers in lockfree list nodes,
+    // this contains almost none of the special cases that gencgc does.
+    if (!layout) return;
+    gc_mark_obj(layout);
+    if (lockfree_list_node_layout_p(LAYOUT(layout))) { // allow untagged 'next'
+        struct instance* node = (struct instance*)where;
+        lispobj next = node->slots[INSTANCE_DATA_START];
+        // ignore if 0
+        if (fixnump(next) && next) __mark_obj(next|INSTANCE_POINTER_LOWTAG);
+    }
+    struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
+    int i;
+    lispobj* slots = where+1;
+    for (i=0; i<nslots; ++i)
+        if (bitmap_logbitp(i, bitmap) && is_lisp_pointer(slots[i]))
+            __mark_obj(slots[i]);
+}
+
 static void trace_object(lispobj* where)
 {
     lispobj header = *where;
     int widetag = header_widetag(header);
+
+    switch (widetag) {
+    case INSTANCE_WIDETAG:
+        return trace_using_layout(instance_layout(where),
+                                  where, instance_length(header));
+    case FUNCALLABLE_INSTANCE_WIDETAG:
+        return trace_using_layout(funinstance_layout(where),
+                                  where, HeaderValue(header) & SHORT_HEADER_MAX_WORDS);
+    }
     sword_t scan_from = 1;
     sword_t scan_to = sizetab[widetag](where);
     sword_t i;
     struct weak_pointer *weakptr;
-    lispobj layout, bitmap;
-
-    /* If the C compiler emits this switch as a jump table, order doesn't matter.
-     * But if as consecutive tests, instance and vector should be tested first
-     * as they are the most freequent */
     switch (widetag) {
-    case INSTANCE_WIDETAG:
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-    /* No need to deal with FINs for non-compact header, because the layout
-       pointer isn't in the header word, the trampoline pointer can only point
-       to readonly space, and all slots are tagged. */
-    case FUNCALLABLE_INSTANCE_WIDETAG:
-        layout = instance_layout(where);
-        gc_mark_obj(layout);
-#else
-        layout = instance_layout(where); // will be marked as where[1]
-#endif
-        if (!layout) break; // fall into general case
-        // mixed boxed/unboxed objects
-        bitmap = LAYOUT(layout)->bitmap;
-        // If no raw slots, just scan without use of the bitmap.
-        // A bitmap of -1 implies that not only are all slots tagged,
-        // there is no special GC method for any slot.
-        if (bitmap == make_fixnum(-1)) break;
-        // Otherwise, the first slot might merit special treatment.
-        if (lockfree_list_node_layout_p(LAYOUT(layout))) {
-            struct instance* node = (struct instance*)where;
-            lispobj next = node->slots[INSTANCE_DATA_START];
-            if (fixnump(next) && next) // ignore initially 0 heap words
-                __mark_obj(next|INSTANCE_POINTER_LOWTAG);
-        }
-        for(i=1; i<scan_to; ++i)
-            if (layout_bitmap_logbitp(i-1, bitmap) && is_lisp_pointer(where[i]))
-                __mark_obj(where[i]);
-        return; // do not scan slots
     case SIMPLE_VECTOR_WIDETAG:
         // non-weak hashtable kv vectors are trivial in fullcgc. Keys don't move
         // so the table will not need rehash as a result of gc.
@@ -387,7 +382,7 @@ void execute_full_mark_phase()
     struct rusage before, after;
     getrusage(RUSAGE_SELF, &before);
 #endif
-    lispobj* where = (lispobj*)STATIC_SPACE_START;
+    lispobj* where = (lispobj*)STATIC_SPACE_OBJECTS_START;
     lispobj* end = static_space_free_pointer;
     while (where < end) {
         lispobj obj = compute_lispobj(where);
@@ -485,7 +480,7 @@ static void sweep_fixedobj_pages(long *zeroed)
         int obj_spacing = fixedobj_page_obj_align(page);
         if (!obj_spacing)
             continue;
-        int nwords = fixedobj_page_obj_size(page);
+        int nwords = obj_spacing >> WORD_SHIFT;
         lispobj *limit = (lispobj*)((char*)obj + IMMOBILE_CARD_BYTES - obj_spacing);
         for ( ; obj <= limit ; obj = (lispobj*)((char*)obj + obj_spacing) ) {
             lispobj header = *obj;
@@ -509,27 +504,16 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
 
     // TODO: consecutive dead objects on same page should be merged.
     for ( ; where < end ; where += nwords ) {
-        lispobj header = *where;
-        if (is_cons_half(header)) {
-            nwords = 2;
-            if (!cons_markedp((lispobj)where)) {
-                if (where[0] | where[1]) {
-               cons:
-                    gc_dcheck(!immobile_space_p((lispobj)where));
-                    NOTE_GARBAGE(page_table[find_page_index(where)].gen,
-                                 where, 2, zeroed,
-                                 where[0] = where[1] = 0);
-                }
-            }
-        } else {
-            nwords = sizetab[header_widetag(header)](where);
+        lispobj word = *where;
+        if (is_header(word)) {
+            nwords = sizetab[header_widetag(word)](where);
             lispobj markbit = MARK_BIT;
-            switch (header_widetag(header)) {
+            switch (header_widetag(word)) {
             case BIGNUM_WIDETAG: markbit = BIGNUM_MARK_BIT; break;
             case FDEFN_WIDETAG : markbit = FDEFN_MARK_BIT; break;
             }
-            if (header & markbit)
-                *where = header ^ markbit;
+            if (word & markbit)
+                *where = word ^ markbit;
             else {
                 // Turn the object into either a (0 . 0) cons
                 // or an unboxed filler depending on size.
@@ -549,6 +533,17 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
                     })
                 }
             }
+        } else {
+            nwords = 2;
+            if (!cons_markedp((lispobj)where)) {
+                if (where[0] | where[1]) {
+               cons:
+                    gc_dcheck(!immobile_space_p((lispobj)where));
+                    NOTE_GARBAGE(page_table[find_page_index(where)].gen,
+                                 where, 2, zeroed,
+                                 where[0] = where[1] = 0);
+                }
+            }
         }
     }
     return 0;
