@@ -19,8 +19,6 @@
 #include "immobile-space.h"
 #include "code.h"
 
-#define CUSTOM_GC_SCAVENGE_FLAG 0x800000
-
 #ifdef LISP_FEATURE_GENCGC
 #include "gencgc-alloc-region.h"
 static inline void *
@@ -48,14 +46,12 @@ extern void *gc_general_alloc(sword_t nbytes,int page_type_flag);
 #define note_transported_object(old, new) /* do nothing */
 
 static inline lispobj
-gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
+gc_general_copy_object(lispobj object, size_t nwords, int page_type_flag)
 {
-    lispobj *new;
-
     CHECK_COPY_PRECONDITIONS(object, nwords);
 
     /* Allocate space. */
-    new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
 
     /* Copy the object. */
     memcpy(new,native_pointer(object),nwords*N_WORD_BYTES);
@@ -65,6 +61,19 @@ gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
     return make_lispobj(new, lowtag_of(object));
 }
 
+// Like above but copy potentially fewer words than are allocated.
+// ('old_nwords' can be, but does not have to be, smaller than 'nwords')
+static inline lispobj
+gc_copy_object_resizing(lispobj object, long nwords, int page_type_flag,
+                        int old_nwords)
+{
+    CHECK_COPY_PRECONDITIONS(object, nwords);
+    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    memcpy(new, native_pointer(object), old_nwords*N_WORD_BYTES);
+    note_transported_object(object, new);
+    return make_lispobj(new, lowtag_of(object));
+}
+
 extern sword_t (*const scavtab[256])(lispobj *where, lispobj object);
 extern struct cons *weak_vectors; /* in gc-common.c */
 extern struct hash_table *weak_hash_tables; /* in gc-common.c */
@@ -225,23 +234,41 @@ static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
      * In cheneygc, chaining is performed in 'trans_weak_pointer'
      * which works just as well, since an object is transported
      * at most once per GC cycle */
-    wp->next = (struct weak_pointer *)LOW_WORD(weak_pointer_chain);
+    wp->next = weak_pointer_chain;
     weak_pointer_chain = wp;
 }
 
-/// Same as Lisp LOGBITP, except no negative bignums allowed.
-static inline boolean layout_bitmap_logbitp(int index, lispobj bitmap)
+#include "genesis/layout.h"
+struct bitmap { sword_t *bits; unsigned int nwords; };
+static inline struct bitmap get_layout_bitmap(struct layout* layout)
 {
-    if (fixnump(bitmap))
-      return (index < (N_WORD_BITS - N_FIXNUM_TAG_BITS))
-          ? (bitmap >> (index+N_FIXNUM_TAG_BITS)) & 1
-          : (sword_t)bitmap < 0;
-    return positive_bignum_logbitp(index, (struct bignum*)native_pointer(bitmap));
+    struct bitmap bitmap;
+    bitmap.bits = (sword_t*)((char*)layout + sizeof (struct layout));
+    // The 2 bits for stable address-based hashing can't ever bet set.
+    bitmap.nwords = (((unsigned int)layout->header >> INSTANCE_LENGTH_SHIFT) & 0x3FFF)
+                    /* subtract one from the struct length in lispwords
+                     * to get the minimum payload length excluding bitmap words. */
+                    - ((sizeof (struct layout) / N_WORD_BYTES) - 1);
+    return bitmap;
+}
+
+/* Return true if the INDEXth bit is set in BITMAP.
+ * Index 0 corresponds to the word just after the instance header.
+ * So index 0 may be the layout pointer if #-compact-instance-header,
+ * or a user data slot if #+compact-instance-header
+ */
+static inline boolean bitmap_logbitp(unsigned int index, struct bitmap bitmap)
+{
+    unsigned int word_index = index / N_WORD_BITS;
+    unsigned int bit_index  = index % N_WORD_BITS;
+    if (word_index >= bitmap.nwords) return bitmap.bits[bitmap.nwords-1] < 0;
+    return (bitmap.bits[word_index] >> bit_index) & 1;
 }
 
 /* Keep in sync with 'target-hash-table.lisp' */
+#define hashtable_kind(ht) ((ht->flags >> (4+N_FIXNUM_TAG_BITS)) & 3)
 #define hashtable_weakp(ht) (ht->flags & (8<<N_FIXNUM_TAG_BITS))
-#define hashtable_weakness(ht) (ht->flags >> (4+N_FIXNUM_TAG_BITS))
+#define hashtable_weakness(ht) (ht->flags >> (6+N_FIXNUM_TAG_BITS))
 
 #if defined(LISP_FEATURE_GENCGC)
 
@@ -324,16 +351,63 @@ static inline void protect_page(void* page_addr, page_index_t page_index)
 #define KV_PAIRS_HIGH_WATER_MARK(kvv) fixnum_value(kvv[0])
 #define KV_PAIRS_REHASH(kvv) kvv[1]
 
-#include "genesis/layout.h"
-static inline int lockfree_list_node_layout_p(struct layout* layout)
+extern lispobj layout_of_layout;
+
+static inline int lockfree_list_node_layout_p(struct layout* layout) {
+    return layout->flags & flag_LockfreeListNode;
+}
+
+/* This is NOT the same value that lisp's %INSTANCE-LENGTH returns.
+ * Lisp always uses the logical length (as originally allocated),
+ * except when heap-walking which requires exact physical sizes */
+static inline int instance_length(lispobj header)
 {
-    // Test +custom-gc-scavenge-flag+ in the _bits or flags depending.
-    // This is a raw slot in either case.
-#ifdef LISP_FEATURE_64_BIT
-    return layout->_bits & 1;
+    // * Byte 3 of an instance header word holds the immobile gen# and visited bit,
+    //   so those have to be masked off.
+    // * fullcgc uses bit index 31 as a mark bit, so that has to
+    //   be cleared. Lisp does not have to clear bit 31 because fullcgc does not
+    //   operate concurrently.
+    // * If the object is in hashed-and-moved state and the original instance payload
+    //   length was odd (total object length was even), then add 1.
+    //   This can be detected by ANDing some bits, bit 10 being the least-significant
+    //   bit of the original size, and bit 9 being the 'hashed+moved' bit.
+    // * 64-bit machines do not need 'long' right-shifts, so truncate to int.
+
+    int extra = ((unsigned int)header >> 10) & ((unsigned int)header >> 9) & 1;
+    return (((unsigned int)header >> INSTANCE_LENGTH_SHIFT) & 0x3FFF) + extra;
+}
+
+/// instance_layout() and layout_of() macros takes a lispobj* and are lvalues
+#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
+
+# ifdef LISP_FEATURE_LITTLE_ENDIAN
+#  define instance_layout(native_ptr) ((uint32_t*)(native_ptr))[1]
+# else
+#  error "No instance_layout() defined"
+# endif
+# define funinstance_layout(native_ptr) instance_layout(native_ptr)
+// generalize over either metatype, but not as general as SB-KERNEL:LAYOUT-OF
+# define layout_of(native_ptr) instance_layout(native_ptr)
+
 #else
-    return layout->flags & 1;
+
+// first 2 words of ordinary instance are: header, layout
+# define instance_layout(native_ptr) ((lispobj*)native_ptr)[1]
+// first 4 words of funcallable instance are: header, trampoline, layout, fin-fun
+# define funinstance_layout(native_ptr) ((lispobj*)native_ptr)[2]
+# define layout_of(native_ptr) \
+  ((lispobj*)native_ptr)[1+((widetag_of(native_ptr)>>LAYOUT_SELECTOR_BIT)&1)]
+
 #endif
+
+/// Return true if 'thing' is a layout.
+static inline boolean layoutp(lispobj thing)
+{
+    lispobj base_ptr = thing - INSTANCE_POINTER_LOWTAG;
+    lispobj layout;
+    if ((base_ptr & LOWTAG_MASK) || !(layout = layout_of((lispobj*)base_ptr)))
+        return 0;
+    return layout == layout_of_layout;
 }
 
 #endif /* _GC_PRIVATE_H_ */
