@@ -178,7 +178,7 @@ static void sigmask_logandc(sigset_t *dest, const sigset_t *source)
  * work for SIGSEGV and similar. It is good enough for timers, and
  * maybe all deferrables. */
 
-#ifdef LISP_FEATURE_DARWIN
+#if defined LISP_FEATURE_DARWIN && defined LISP_FEATURE_SB_THREAD
 pthread_key_t sigwait_bug_mitigation;
 int sigwait_bug_mitigation_count;
 #endif
@@ -189,7 +189,7 @@ int sigwait_bug_mitigation_count;
 static void
 resignal_to_lisp_thread(int signal, os_context_t *context)
 {
-#ifdef LISP_FEATURE_DARWIN
+#if defined LISP_FEATURE_DARWIN && defined LISP_FEATURE_SB_THREAD
     if (signal == SIG_STOP_FOR_GC && pthread_getspecific(sigwait_bug_mitigation)) {
         // fprintf(stderr, "ignored a STOP_FOR_GC signal\n");
         __sync_fetch_and_add(&sigwait_bug_mitigation_count, 1);
@@ -253,9 +253,8 @@ resignal_to_lisp_thread(int signal, os_context_t *context)
  * interrupt_handle_now_handler
  * maybe_now_maybe_later
  * low_level_handle_now_handler
- * low_level_maybe_now_maybe_later
  *
- * This gives us a single point of control (or four) over errno, fp
+ * This gives us a single point of control (or three) over errno, fp
  * control word, and fixing up signal context on sparc.
  *
  * The SPARC/Linux platform doesn't quite do signals the way we want
@@ -291,13 +290,6 @@ static void store_signal_data_for_later (struct interrupt_data *data,
 
 /* Generic signal related utilities. */
 
-void
-get_current_sigmask(sigset_t *sigset)
-{
-    /* Get the current sigmask, by blocking the empty set. */
-    thread_sigmask(SIG_BLOCK, 0, sigset);
-}
-
 // Stringify sigset into the supplied result buffer.
 void
 sigset_tostring(const sigset_t *sigset, char* result, int result_length)
@@ -327,7 +319,7 @@ all_signals_blocked_p(const sigset_t *sigset, sigset_t *sigset2,
     boolean has_blocked = 0, has_unblocked = 0;
     sigset_t current;
     if (sigset == 0) {
-        get_current_sigmask(&current);
+        thread_sigmask(SIG_BLOCK, 0, &current);
         sigset = &current;
     }
     for(i = 1; i <= MAX_SIGNUM; i++) {
@@ -359,7 +351,6 @@ sigaddset_deferrable(sigset_t *s)
     sigaddset(s, SIGINT);
     sigaddset(s, SIGTERM);
     sigaddset(s, SIGQUIT);
-    sigaddset(s, SIGPIPE);
     sigaddset(s, SIGALRM);
     sigaddset(s, SIGURG);
     sigaddset(s, SIGTSTP);
@@ -391,6 +382,11 @@ sigaddset_blockable(sigset_t *sigset)
 {
     sigaddset_deferrable(sigset);
     sigaddset_gc(sigset);
+    // SIGPIPE is *NOT* an asynchronous signal. In normal usage you receive this signal
+    // synchronously in response to a system call. As such we do not place it in the
+    // deferrable set, but it _is_ blockable. If you're doing something wherein SIGPIPE
+    // interrupts a pseudo-atomic section, then you're doing something wrong for sure.
+    sigaddset(sigset, SIGPIPE);
 }
 
 /* initialized in interrupt_init */
@@ -440,8 +436,7 @@ check_blockables_unblocked_or_lose(sigset_t *sigset)
 #endif
 }
 
-void
-check_blockables_blocked_or_lose(sigset_t *sigset)
+static void assert_blockables_blocked()
 {
 #if !defined(LISP_FEATURE_WIN32)
     /* On Windows, there are no actual signals, but since the win32 port
@@ -462,7 +457,7 @@ check_blockables_blocked_or_lose(sigset_t *sigset)
      * So we merely skip this assertion.
      *   -- DFL, trying to expand on a comment by AK.
      */
-    if (!blockables_blocked_p(sigset))
+    if (!blockables_blocked_p(0))
         lose("blockables unblocked");
 #endif
 }
@@ -584,7 +579,7 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
 {
 #ifndef LISP_FEATURE_WIN32
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
     sigset_t oldset;
     /* Obviously, this function is called when signals may not be
      * blocked. Let's make sure we are not interrupted. */
@@ -641,7 +636,7 @@ check_interrupt_context_or_lose(os_context_t *context)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
     int interrupt_deferred_p = (data->pending_handler != 0);
     int interrupt_pending = (read_TLS(INTERRUPT_PENDING,thread) != NIL);
     sigset_t *sigset = os_context_sigmask_addr(context);
@@ -781,15 +776,11 @@ build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
 
 /* Stores the context for gc to scavange and builds fake stack
  * frames. */
-void
-fake_foreign_function_call(os_context_t *context)
+void fake_foreign_function_call_noassert(os_context_t *context)
 {
     int context_index;
     struct thread *thread=arch_os_get_current_thread();
 
-    /* context_index incrementing must not be interrupted */
-    check_blockables_blocked_or_lose(0);
-
 #ifdef reg_ALLOC
 #ifdef LISP_FEATURE_SB_THREAD
     thread->pseudo_atomic_bits =
@@ -800,17 +791,11 @@ fake_foreign_function_call(os_context_t *context)
             (*os_context_register_addr(context, reg_ALLOC));
 /*     fprintf(stderr,"dynamic_space_free_pointer: %p\n", */
 /*             dynamic_space_free_pointer); */
-#if defined(LISP_FEATURE_ALPHA) || defined(LISP_FEATURE_MIPS)
+#if defined(LISP_FEATURE_MIPS)
     if ((sword_t)dynamic_space_free_pointer & 1) {
         lose("dead in fake_foreign_function_call, context = %x", context);
     }
 #endif
-/* why doesnt PPC and SPARC do something like this: */
-#if defined(LISP_FEATURE_HPPA)
-    if ((sword_t)dynamic_space_free_pointer & 4) {
-        lose("dead in fake_foreign_function_call, context = %x, d_s_f_p = %x", context, dynamic_space_free_pointer);
-    }
-#endif
 #endif
 #ifdef reg_BSP
     set_binding_stack_pointer(thread,
@@ -831,10 +816,8 @@ fake_foreign_function_call(os_context_t *context)
     context_index =
         fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
-    if (context_index >= (MAX_INTERRUPTS-THREAD_HEADER_SLOTS)) {
-        lose("maximum interrupt nesting depth (%d) exceeded",
-             MAX_INTERRUPTS-THREAD_HEADER_SLOTS);
-    }
+    if (context_index >= MAX_INTERRUPTS)
+        lose("maximum interrupt nesting depth (%d) exceeded", MAX_INTERRUPTS);
 
     bind_variable(FREE_INTERRUPT_CONTEXT_INDEX,
                   make_fixnum(context_index + 1),thread);
@@ -848,6 +831,12 @@ fake_foreign_function_call(os_context_t *context)
     foreign_function_call_active_p(thread) = 1;
 #endif
 }
+void fake_foreign_function_call(os_context_t *context)
+{
+    /* context_index incrementing must not be interrupted */
+    assert_blockables_blocked();
+    fake_foreign_function_call_noassert(context);
+}
 
 /* blocks all blockable signals.  If you are calling from a signal handler,
  * the usual signal mask will be restored from the context when the handler
@@ -950,8 +939,7 @@ interrupt_internal_error(os_context_t *context, boolean continuable)
 boolean
 interrupt_handler_pending_p(void)
 {
-    struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(arch_os_get_current_thread());
     return (data->pending_handler != 0);
 }
 
@@ -977,7 +965,7 @@ interrupt_handle_pending(os_context_t *context)
      * pending asynchronous tasks. */
 
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
 
     if (arch_pseudo_atomic_atomic(context)) {
         lose("Handling pending interrupt in pseudo atomic.");
@@ -985,7 +973,7 @@ interrupt_handle_pending(os_context_t *context)
 
     FSHOW_SIGNAL((stderr, "/entering interrupt_handle_pending\n"));
 
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
 #ifndef LISP_FEATURE_SB_SAFEPOINT
     /*
      * (On safepoint builds, there is no gc_blocked_deferrables nor
@@ -1083,7 +1071,7 @@ interrupt_handle_pending(os_context_t *context)
             lose("Trapping to run pending handler while GC in progress.");
         }
 
-        check_blockables_blocked_or_lose(0);
+        assert_blockables_blocked();
 
         /* No GC shall be lost. If SUB_GC triggers another GC then
          * that should be handled on the spot. */
@@ -1149,7 +1137,7 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
 
     if (!functionp(handler)) return;
 
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
 
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     if (sigismember(&deferrable_sigset,signal))
@@ -1159,7 +1147,9 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
     were_in_lisp = !foreign_function_call_active_p(arch_os_get_current_thread());
     if (were_in_lisp)
     {
-        fake_foreign_function_call(context);
+        // Use the variant of fake_ffc that doesn't do another pthread_sigmask syscall,
+        // as we've just asserted that signals are blocked.
+        fake_foreign_function_call_noassert(context);
     }
 
         /* Once we've decided what to do about contexts in a
@@ -1221,11 +1211,11 @@ can_handle_now(void *handler, struct interrupt_data *data,
 {
     struct thread *thread=arch_os_get_current_thread();
 
-    check_blockables_blocked_or_lose(0);
+    assert_blockables_blocked();
 
     if (read_TLS(INTERRUPT_PENDING,thread) != NIL)
         lose("interrupt already pending");
-    if (thread->interrupt_data->pending_handler)
+    if (thread_interrupt_data(thread).pending_handler)
         lose("there is a pending handler already (PA)");
     if (data->gc_blocked_deferrables)
         lose("can_handle_now: gc_blocked_deferrables true");
@@ -1305,33 +1295,11 @@ maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
 {
     SAVE_ERRNO(signal,context,void_context);
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
     if (can_handle_now(interrupt_handle_now, data, signal, info, context))
         interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
 }
-
-static void
-low_level_interrupt_handle_now(int signal, siginfo_t *info,
-                               os_context_t *context)
-{
-    /* No FP control fixage needed, caller has done that. */
-    check_blockables_blocked_or_lose(0);
-    check_interrupts_enabled_or_lose(context);
-    (*interrupt_low_level_handlers[signal])(signal, info, context);
-}
-
-static void
-low_level_maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-    struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
-
-    if (can_handle_now(low_level_interrupt_handle_now, data, signal, info, context))
-        low_level_interrupt_handle_now(signal, info, context);
-    RESTORE_ERRNO;
-}
 #endif
 
 #ifdef THREADS_USING_GCSIGNAL
@@ -1384,21 +1352,24 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * pseudo_atomic_interrupted but without a pending interrupt or
      * GC. GC_BLOCKED_DEFERRABLES is also left at 1. So let's tidy it
      * up. */
-    if (thread->interrupt_data->gc_blocked_deferrables) {
+    if (thread_interrupt_data(thread).gc_blocked_deferrables) {
         FSHOW_SIGNAL((stderr,"cleaning up after gc_blocked_deferrables\n"));
         clear_pseudo_atomic_interrupted(thread);
-        sigcopyset(os_context_sigmask_addr(context),
-                   &thread->interrupt_data->pending_mask);
-        thread->interrupt_data->gc_blocked_deferrables = 0;
+        struct interrupt_data *interrupt_data = &thread_interrupt_data(thread);
+        sigcopyset(os_context_sigmask_addr(context), &interrupt_data->pending_mask);
+        interrupt_data->gc_blocked_deferrables = 0;
     }
 
-    if(thread_state(thread)!=STATE_RUNNING) {
-        // macOS warns if OBJ_FMTX is used to format a 'sword_t'
-        // which fixnum_value() returns.
-        lose("sig_stop_for_gc_handler: wrong thread state: %"OBJ_FMTX,
-             (lispobj)fixnum_value(thread->state));
-    }
+    /* No need to use an atomic memory load here - this thead "owns" its state
+     * for now, and nobody else touches it, the sole exception being that GC
+     * sets it to RUNNING. The loads inside thread_wait_until_not()
+     * are slightly more interesting from that perspective */
+    if (thread->state != STATE_RUNNING)
+        lose("stop_for_gc: bad thread state: %x", (int)thread->state);
 
+    /* We say that the thread is "stopped" as of now, but the blocking operation
+     * occurs below at thread_wait_until_not(STATE_STOPPED). Note that sem_post()
+     * is expressly permitted in signal handlers, and set_thread_state uses it */
     set_thread_state(thread, STATE_STOPPED, 0);
     FSHOW_SIGNAL((stderr,"suspended\n"));
 
@@ -1408,13 +1379,19 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * actually a must. */
     scrub_control_stack();
 
-    wait_for_thread_state_change(thread, STATE_STOPPED);
+    /* Now we wait on a semaphore, which, to be pedantic, is not specified as async-safe.
+     * Normally the way to implement a "suspend" operation is to issue any blocking
+     * syscall such as sigsuspend() or select(). Apparently every OS + C runtime that
+     * we wish to support has no problem with sem_wait() here in the signal handler. */
+    int my_state = thread_wait_until_not(STATE_STOPPED, thread);
     FSHOW_SIGNAL((stderr,"resumed\n"));
 
-    if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %"OBJ_FMTX,
-             (lispobj)fixnum_value(thread_state(thread)));
-    }
+    /* The state can't go from STOPPED to DEAD because it's this thread is reading
+     * its own state, hence it must be running.
+     * (If we tried to observe a different thread, it could appear to change from
+     * STOPPED to DEAD, skipping RUNNING, because if you blink you might miss it) */
+    if (my_state != STATE_RUNNING)
+        lose("stop_for_gc: bad state on wakeup: %x", my_state);
 
     if (was_in_lisp) {
         undo_fake_foreign_function_call(context);
@@ -1509,11 +1486,11 @@ arrange_return_to_c_function(os_context_t *context,
      */
 
 #ifndef LISP_FEATURE_DARWIN
-    u32 *sp=(u32 *)*os_context_register_addr(context,reg_ESP);
+    uint32_t *sp=(uint32_t *)*os_context_register_addr(context,reg_ESP);
 #endif
 
 #if defined(LISP_FEATURE_DARWIN)
-    u32 *register_save_area = (u32 *)os_allocate(0x40);
+    uint32_t *register_save_area = (uint32_t *)os_allocate(0x40);
 
     FSHOW_SIGNAL((stderr, "/arrange_return_to_lisp_function: preparing to go to function %x, sp: %x\n", function,
                   *os_context_register_addr(context,reg_ESP)));
@@ -1545,7 +1522,7 @@ arrange_return_to_c_function(os_context_t *context,
 #else
 
     /* return address for call_into_lisp: */
-    *(sp-15) = (u32)post_signal_tramp;
+    *(sp-15) = (uint32_t)post_signal_tramp;
     *(sp-14) = function;        /* args for call_into_lisp : function*/
     *(sp-13) = 0;               /*                           arg array */
     *(sp-12) = 0;               /*                           no. args */
@@ -1568,10 +1545,10 @@ arrange_return_to_c_function(os_context_t *context,
 #endif
 
 #elif defined(LISP_FEATURE_X86_64)
-    u64 *sp=(u64 *)*os_context_register_addr(context,reg_RSP);
+    uword_t *sp=(uword_t *)*os_context_register_addr(context,reg_RSP);
 
     /* return address for call_into_lisp: */
-    *(sp-18) = (u64)post_signal_tramp;
+    *(sp-18) = (uint64_t)post_signal_tramp;
 
     *(sp-17)=*os_context_register_addr(context,reg_R15);
     *(sp-16)=*os_context_register_addr(context,reg_R14);
@@ -1817,92 +1794,6 @@ extern void restore_sbcl_signals () {
     }
 }
 
-#if defined(LISP_FEATURE_SB_SAFEPOINT_STRICTLY) && !defined(LISP_FEATURE_WIN32)
-
-static void *
-signal_thread_trampoline(void *pthread_arg)
-{
-    intptr_t signo = (intptr_t) pthread_arg;
-    os_context_t fake_context;
-    siginfo_t fake_info;
-#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
-    mcontext_t uc_regs;
-#endif
-
-    memset(&fake_info, 0, sizeof(fake_info));
-    memset(&fake_context, 0, sizeof(fake_context));
-#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
-    memset(&uc_regs, 0, sizeof(uc_regs));
-    fake_context.uc_mcontext.uc_regs = &uc_regs;
-#endif
-
-    *os_context_pc_addr(&fake_context) = (intptr_t) &signal_thread_trampoline;
-#ifdef ARCH_HAS_STACK_POINTER /* aka x86(-64) */
-    *os_context_sp_addr(&fake_context) = (intptr_t) __builtin_frame_address(0);
-#endif
-
-    signal_handler_callback(interrupt_handlers[signo].lisp,
-                            signo, &fake_info, &fake_context);
-    return 0;
-}
-
-static void
-sigprof_handler_trampoline(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-    struct thread *self = arch_os_get_current_thread();
-
-    /* alloc() is not re-entrant and still uses pseudo atomic (even though
-     * inline allocation does not).  In this case, give up. */
-    if (get_pseudo_atomic_atomic(self))
-        goto cleanup;
-
-    struct alloc_region tmp = self->alloc_region;
-    self->alloc_region = self->sprof_alloc_region;
-    self->sprof_alloc_region = tmp;
-
-    interrupt_handle_now_handler(signal, info, void_context);
-
-    /* And we're back.  We know that the SIGPROF handler never unwinds
-     * non-locally, and can simply swap things back: */
-
-    tmp = self->alloc_region;
-    self->alloc_region = self->sprof_alloc_region;
-    self->sprof_alloc_region = tmp;
-
-cleanup:
-    ; /* Dear C compiler, it's OK to have a label here. */
-    RESTORE_ERRNO;
-}
-
-static void
-spawn_signal_thread_handler(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-
-    pthread_attr_t attr;
-    pthread_t th;
-
-    // This is total bullcrap. None of these functions are specified
-    // as being async-signal-safe, yet here were are in a signal handler
-    // trying to create a thread.
-    if (pthread_attr_init(&attr))
-        goto lost;
-    if (pthread_attr_setstacksize(&attr, thread_control_stack_size))
-        goto lost;
-    if (pthread_create(&th, &attr, &signal_thread_trampoline, (void*)(intptr_t) signal))
-        goto lost;
-    if (pthread_attr_destroy(&attr))
-        goto lost;
-
-    RESTORE_ERRNO;
-    return;
-
-lost:
-    lose("spawn_signal_thread_handler");
-}
-#endif
-
 static void
 low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 {
@@ -1912,8 +1803,7 @@ low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 }
 
 void
-ll_install_handler (int signal,
-                                              interrupt_handler_t handler)
+ll_install_handler (int signal, interrupt_handler_t handler)
 {
     struct sigaction sa;
 
@@ -1923,19 +1813,19 @@ ll_install_handler (int signal,
 
     if (ARE_SAME_HANDLER(handler, SIG_DFL))
         sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))handler;
-    else if (sigismember(&deferrable_sigset,signal))
-        sa.sa_sigaction = low_level_maybe_now_maybe_later;
-    else
-        sa.sa_sigaction = low_level_handle_now_handler;
-
+    else if (
 #ifdef LISP_FEATURE_SB_THRUPTION
     /* It's in `deferrable_sigset' so that we block&unblock it properly,
-     * but we don't actually want to defer it.  And if we put it only
+     * but we don't actually want to defer it, at least not here.
+     * (It might get deferred until a safepoint). And if we put it only
      * into blockable_sigset, we'd have to special-case it around thread
      * creation at least. */
-    if (signal == SIGPIPE)
-        sa.sa_sigaction = low_level_handle_now_handler;
+             (signal == SIGURG) ||
 #endif
+             !sigismember(&deferrable_sigset,signal))
+        sa.sa_sigaction = low_level_handle_now_handler;
+    else
+        lose("Can't install low-level handler");
 
     sa.sa_mask = blockable_sigset;
     sa.sa_flags = SA_SIGINFO | SA_RESTART | OS_SA_NODEFER;
@@ -1956,9 +1846,7 @@ ll_install_handler (int signal,
 #endif
 
 /* This is called from Lisp. */
-void
-install_handler(int signal, lispobj handler,
-                int __attribute__((unused)) synchronous)
+void install_handler(int signal, lispobj handler)
 {
 #ifndef LISP_FEATURE_WIN32
     struct sigaction sa;
@@ -1967,7 +1855,7 @@ install_handler(int signal, lispobj handler,
         // Our "abstract" values for SIG_DFL and SIG_IGN are 0 and 1
         // respectively which are probably the real values from signal.h
         // but this way way don't need to put them in grovel-headers.c
-        if (fixnump(handler)) {
+        if (handler== 0 || handler==1) {
             memset(&sa, 0, sizeof sa);
             sa.sa_handler = handler ? SIG_IGN : SIG_DFL;
             // assign the OS level action before clearing the lisp function.
@@ -1978,13 +1866,6 @@ install_handler(int signal, lispobj handler,
             lisp_sig_handlers[signal] = NIL;
             return;
         }
-#ifdef LISP_FEATURE_SB_SAFEPOINT_STRICTLY
-        if (signal == SIGPROF)
-            sa.sa_sigaction = sigprof_handler_trampoline;
-        else if (!synchronous)
-            sa.sa_sigaction = spawn_signal_thread_handler;
-        else
-#endif
         if (sigismember(&deferrable_sigset, signal))
             sa.sa_sigaction = maybe_now_maybe_later;
         else
